@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/include/aide.h,v 1.9 2000/04/20 12:44:48 rammer Exp $
+ * $Header: /cvs-root-aide/aide/include/aide.h,v 1.11 2001/07/10 09:34:48 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,10 +24,12 @@
 #include "db_config.h"
 #include "config.h"
 
+#ifndef __NetBSD__
 #ifndef _POSIX_C_SOURCE
 /* For _POSIX_THREAD_SEMANTICS _REENTRANT */
 #define _POSIX_C_SOURCE 199506L
 #endif /* _POSIX_C_SOURCE */
+#endif /* __NetBSD__ */
 
 
 #define AIDEVERSION VERSION
@@ -39,7 +41,7 @@ extern db_config* conf;
 
 void print_version(void);
 
-void usage(void);
+void usage(int);
 
 int read_param(int argc,char**argv);
 
