@@ -1,6 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999 Rami Lehti, Pablo Virolainen
+ * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
+ * $Header: /cvs-root-aide/aide/include/be.h,v 1.6 2000/04/20 12:44:49 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +22,6 @@
 #define _BE_H_INCLUDED
 #include "db_config.h"
 
-FILE* be_init(int inout,url_t* u);
+FILE* be_init(int inout,url_t* u,int iszipped);
 
 #endif /* _BE_H_INCLUDED */
