@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/include/db.h,v 1.7 2000/04/20 12:44:50 rammer Exp $
+ * $Header: /cvs-root-aide/aide/include/db.h,v 1.8 2000/07/28 12:32:20 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,7 +34,7 @@ int db_writeline(db_line*,db_config*);
 
 int db_close(db_config*);
 
-extern char* db_names[];
+extern const char* db_names[];
 extern const int db_value[];
 
 #endif
