@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/include/db_file.h,v 1.6 2000/04/20 12:44:51 rammer Exp $
+ * $Header: /cvs-root-aide/aide/include/db_file.h,v 1.8 2002/02/10 19:10:32 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,7 +28,7 @@ extern long db_lineno;
 char** db_readline_file(db_config*);
 int db_writespec_file(db_config*);
 int db_writeline_file(db_line* line,db_config* conf);
+int db_close_file(db_config* conf);
 void handle_gzipped_input(int out);
-void handle_gzipped_output(int in);
 
 #endif
