@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000 Rami Lehti,Pablo Virolainen
- * $Header: /cvs-root-aide/aide/include/seltree.h,v 1.8 2000/04/20 12:44:53 rammer Exp $
+ * $Header: /cvs-root-aide/aide/include/seltree.h,v 1.9 2002/02/10 17:31:29 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -37,8 +37,13 @@ typedef struct seltree {
   list* equ_rx_lst;
   list* childs;
   struct seltree* parent;
+
   char* path;
   int checked;
+
+  long      conf_lineno;
+  char*     rx;    
+
 } seltree;
 
 #endif /* _SELTREE_H_INCLUDED*/
