@@ -2,7 +2,7 @@
 **
 ** Copyright (C) 1994 Swedish University Network (SUNET)
 ** Modified by Rami Lehti (C) 1999
-** $Header: /cvs-root-aide/aide/src/base64.c,v 1.11 2000/04/20 12:44:54 rammer Exp $
+** $Header: /cvs-root-aide/aide/src/base64.c,v 1.12 2000/05/29 08:38:55 rammer Exp $
 **
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -36,6 +36,9 @@
 #include "config.h"
 #include "base64.h"
 #include "report.h"
+/*for locale support*/
+#include "locale-aide.h"
+/*for locale support*/
 
 char tob64[] = 
 "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
