@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000 Rami Lehti,Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/be.c,v 1.12 2000/04/20 12:44:55 rammer Exp $
+ * $Header: /cvs-root-aide/aide/src/be.c,v 1.20 2002/02/10 19:10:32 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,6 +32,75 @@
 #ifdef WITH_PSQL
 #include "libpq-fe.h"
 #endif
+/*for locale support*/
+#include "locale-aide.h"
+/*for locale support*/
+
+#ifdef WITH_PSQL
+int be_sql_readinit(psql_data* ret) {
+  /* Yes.. we don't want to know about two first result.. 
+     and we want no memoryleaking.
+  */
+  int i,j,nFields;
+  char* s;
+  char declare []="DECLARE aidecursor CURSOR FOR select * from ";
+  
+  s = (char*)malloc(strlen(declare)+strlen(ret->table)+1);
+  s[0]=0;
+  s=strcat(s,declare);
+  s=strcat(s,ret->table);
+  
+  ret->res=PQexec(ret->conn,s);
+  /*
+  PQclear(ret->res);
+  ret->res=PQexec(ret->conn,
+		  ret->table
+		  );
+  */
+		  
+  if (!ret->res || PQresultStatus(ret->res) != PGRES_COMMAND_OK) {
+    
+    if (ret->res!=NULL) {
+      error(255,"Psql error: %s\n",PQresStatus(PQresultStatus(ret->res)));
+      PQclear(ret->res);
+    }
+    return RETFAIL;
+  }
+  PQclear(ret->res);
+  
+  ret -> res = PQexec(ret->conn, "FETCH ALL in aidecursor");
+  
+  if (!ret->res || PQresultStatus(ret->res) != PGRES_TUPLES_OK)
+    {
+      fprintf(stderr, "FETCH ALL command didn't return tuples properly\n");
+      PQclear(ret->res);
+      abort();
+    }
+  
+  
+  /* first, print out the attribute names */
+  nFields = PQnfields(ret->res);
+  for (i = 0; i < nFields; i++)
+    printf("%-15s", PQfname(ret->res, i));
+  printf("\n\n");
+  
+  
+  for(i=0;i<db_unknown;i++){
+    ret->des[i]=PQfnumber(ret->res,db_names[i]);
+    if (ret->des[i]!=-1) {
+      printf("Kentt %i,%s \n",ret->des[i],db_names[i]);
+    }
+  }
+  
+  ret->curread=0;
+  ret->maxread=PQntuples(ret->res);
+  /* And now we know how many fields we have.. */
+  
+  error(0,"%i tuplesi\n",ret->maxread);
+  
+  return RETOK;
+  
+}
 
 char* get_first_value(char** in){
   int i=0;
@@ -51,78 +120,108 @@ char* get_first_value(char** in){
   return ret;
 }
 
+#endif
+
 void* be_init(int inout,url_t* u,int iszipped)
 {
-  FILE* fh=NULL;
+  void* fh=NULL;
   long a=0;
   char* err=NULL;
-  int retval;
 
-#ifdef WITH_ZLIB
-  if(iszipped) {
-    int pipefd[2];
-    
-    if(pipe(pipefd)<0){
-      error(0,"Couldn't open a pipe for zipped output\n");
-      return NULL;
-    }
-    if((retval=fork())==0){
-      /* child process */
-      close(pipefd[1]);
-      fh=be_init(inout,u,0);
-      conf->db_gzout=gzdopen(fileno(fh),"wb");
-      handle_gzipped_output(pipefd[0]);
-    } else {
-      /* parent process */
-      close(pipefd[0]);
-      fh=fdopen(pipefd[1],"w");
-    }
-    return fh;
-  }
-#endif
-  
   switch (u->type) {
   case url_file : {
-    error(200,"Opening file \"%s\" for %s\n",u->value,inout?"r":"w+");
-    fh=fopen(u->value,inout?"r":"w+");
-    if(fh==NULL){
-      error(0,"Couldn't open file %s for %s",u->value,
-	    inout?"reading\n":"writing\n");
+    error(200,_("Opening file \"%s\" for %s\n"),u->value,inout?"r":"w+");
+#ifdef WITH_ZLIB
+    if(iszipped && !inout){
+      fh=gzopen(u->value,"wb+");
+      if(fh==NULL){
+	error(0,_("Couldn't open file %s for %s"),u->value,
+	      inout?"reading\n":"writing\n");
+      }
     }
+    else{
+#endif
+      fh=fopen(u->value,inout?"r":"w+");
+      if(fh==NULL){
+	error(0,_("Couldn't open file %s for %s"),u->value,
+	      inout?"reading\n":"writing\n");
+      }
+#ifdef WITH_ZLIB
+    }
+#endif
     return fh;
-  }
+    }
   case url_stdout : {
+#ifdef WITH_ZLIB
+    if(iszipped){
+      return gzdopen(fileno(stdout),"wb");
+    }
+    else{
+#endif
     return stdout;
+#ifdef WITH_ZLIB
+    }
+#endif
   }
   case url_stdin : {
-    return stdin;
+#ifdef WITH_ZLIB
+    if(iszipped){
+      return gzdopen(fileno(stdin),"r");
+    }
+    else{
+#endif
+      return stdin;
+#ifdef WITH_ZLIB
+    }
+#endif
   }
   case url_stderr : {
-    return stderr;
+#ifdef WITH_ZLIB
+    if(iszipped){
+      return gzdopen(fileno(stderr),"wb");
+    }
+    else{
+#endif
+      return stderr;
+#ifdef WITH_ZLIB
+    }
+#endif
   }
   case url_fd : {
     a=strtol(u->value,&err,10);
     if(*err!='\0'||errno==ERANGE){
       error(0,"Illegal file descriptor value:%s\n",u->value);
     }
-    fh=fdopen(a,inout?"r":"w");
-    if(fh==NULL){
-      error(0,"Couldn't reopen file descriptor %i\n",a);
+#ifdef WITH_ZLIB
+    if(iszipped && !inout){
+      fh=gzdopen(a,"w");
+      if(fh==NULL){
+	error(0,"Couldn't reopen file descriptor %i\n",a);
+      }
     }
+    else{
+#endif
+      fh=fdopen(a,inout?"r":"w");
+      if(fh==NULL){
+	error(0,"Couldn't reopen file descriptor %i\n",a);
+      }
+#ifdef WITH_ZLIB
+    }
+#endif
     return fh;
   }
 #ifdef WITH_PSQL
   case url_sql : {
     char *pghost, *pgport, *pgoptions, *pgtty, *dbName, *login, *pwd;
     char *tmp,*tmp2;
+    
     psql_data* ret = (psql_data*) malloc(sizeof(psql_data)*1);
-
+    
     if (ret==NULL) {
       error(0,"Not enough memory for postgres sql connection\n");
       return ret;
     }
-
-
+    
     tmp=strdup(u->value);
     tmp2=tmp;
     
@@ -197,7 +296,24 @@ void* be_init(int inout,url_t* u,int iszipped)
        be freeed 
     */
     ret->table = strdup(ret->table);
-
+    
+    /* And now we have made a connection to database.. 
+       Next thing we do is to begin a new transaction block */
+    
+    ret->res = PQexec(ret->conn, "BEGIN");
+    
+    if (!ret->res || PQresultStatus(ret->res) != PGRES_COMMAND_OK) {
+      error(0,"BEGIN command failed... ");
+      PQclear(ret->res);
+      free(ret);
+      ret=NULL;
+    } else {
+      PQclear(ret->res);
+      if ((inout?be_sql_readinit(ret):RETOK)!=RETOK) {
+	error(255,"jotain meni pieleeen.. sql");
+	return NULL;
+      }
+    }
     free(tmp2);
     return ret;
   }
