@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/commandconf.c,v 1.40 2000/04/20 12:44:55 rammer Exp $
+ * $Header: /cvs-root-aide/aide/src/commandconf.c,v 1.46 2002/02/10 17:31:29 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,6 +32,9 @@
 #include "gen_list.h"
 #include "symboltable.h"
 #include "util.h"
+/*for locale support*/
+#include "locale-aide.h"
+/*for locale support*/
 
 
 #define BUFSIZE 4096
@@ -57,6 +60,7 @@ int commandconf(const char mode,const char* line)
     else {
       tmp=(char*)malloc(sizeof(char)
 			*(strlen(before)+strlen(line)+2));
+      tmp[0]='\0';
       strcat(tmp,before);
       strcat(tmp,"\n");
       strcat(tmp,line);
@@ -89,19 +93,19 @@ int commandconf(const char mode,const char* line)
     int rv=-1;
     
     if (config!=NULL && strcmp(config,"-")==0) {
-      error(255,"Config from stdin\n");
+      error(255,_("Config from stdin\n"));
       rv=0;
     } else {
       
       rv=access(config,R_OK);
       if(rv==-1){
-	error(0,"Cannot access config file:%s:%s\n",config,strerror(errno));
+	error(0,_("Cannot access config file:%s:%s\n"),config,strerror(errno));
       }
     }
     
     if(before==NULL&&after==NULL&&
        (config==NULL||strcmp(config,"")==0||rv==-1)){
-      error(0,"No config defined\n");
+      error(0,_("No config defined\n"));
       return RETFAIL;
     }
     if(before!=NULL) {
@@ -141,7 +145,7 @@ int commandconf(const char mode,const char* line)
     break;
   }
   default: {
-    error(0,"Illegal argument %c to commmandconf()\n",mode);
+    error(0,_("Illegal argument %c to commmandconf()\n"),mode);
     break;
   }
   }
@@ -161,67 +165,78 @@ int check_db_order(DB_FIELD* d,int size, DB_FIELD a)
 int check_dboo(DB_FIELD a){
   return check_db_order(conf->db_out_order,conf->db_out_size,a);
 }
-void update_db_out_order(int a)
+void update_db_out_order(int attr)
 {
-  if((a&DB_PERM) && (check_dboo(db_perm)!=RETFAIL)){
+  if (check_dboo(db_attr)==RETOK) {
+    conf->db_out_order[conf->db_out_size++]=db_attr;
+  }
+  if((attr&DB_PERM) && (check_dboo(db_perm)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_perm;
   }
-  if((a&DB_BCOUNT) && (check_dboo(db_bcount)!=RETFAIL)){
+  if((attr&DB_BCOUNT) && (check_dboo(db_bcount)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_bcount;
   }
-  if((a&DB_UID) && (check_dboo(db_uid)!=RETFAIL)){
+  if((attr&DB_UID) && (check_dboo(db_uid)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_uid;
   }
-  if((a&DB_GID) && (check_dboo(db_gid)!=RETFAIL)){
+  if((attr&DB_GID) && (check_dboo(db_gid)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_gid;
   }
-  if((a&DB_SIZE) && (check_dboo(db_size)!=RETFAIL)){
+  if((attr&DB_SIZE) && (check_dboo(db_size)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_size;
   }
-  if((a&DB_SIZEG) && (check_dboo(db_size)!=RETFAIL)){
+  if((attr&DB_SIZEG) && (check_dboo(db_size)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_size;
   }
-  if((a&DB_ATIME) && (check_dboo(db_atime)!=RETFAIL)){
+  if((attr&DB_ATIME) && (check_dboo(db_atime)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_atime;
   }
-  if((a&DB_MTIME) && (check_dboo(db_mtime)!=RETFAIL)){
+  if((attr&DB_MTIME) && (check_dboo(db_mtime)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_mtime;
   }
-  if((a&DB_CTIME) && (check_dboo(db_ctime)!=RETFAIL)){
+  if((attr&DB_CTIME) && (check_dboo(db_ctime)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_ctime;
   }
-  if((a&DB_INODE) && (check_dboo(db_inode)!=RETFAIL)){
+  if((attr&DB_INODE) && (check_dboo(db_inode)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_inode;
   }
-  if((a&DB_LNKCOUNT) && (check_dboo(db_lnkcount)!=RETFAIL)){
+  if((attr&DB_LNKCOUNT) && (check_dboo(db_lnkcount)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_lnkcount;
   }
-  if((a&DB_MD5) && (check_dboo(db_md5)!=RETFAIL)){
+  if((attr&DB_MD5) && (check_dboo(db_md5)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_md5;
   }
-  if((a&DB_SHA1) && (check_dboo(db_sha1)!=RETFAIL)){
+  if((attr&DB_SHA1) && (check_dboo(db_sha1)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_sha1;
   }
-  if((a&DB_RMD160) && (check_dboo(db_rmd160)!=RETFAIL)){
+  if((attr&DB_RMD160) && (check_dboo(db_rmd160)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_rmd160;
   }
-  if((a&DB_TIGER) && (check_dboo(db_tiger)!=RETFAIL)){
+  if((attr&DB_TIGER) && (check_dboo(db_tiger)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_tiger;
   }
 #ifdef WITH_MHASH
-  if((a&DB_CRC32) && (check_dboo(db_crc32)!=RETFAIL)){
+  if((attr&DB_CRC32) && (check_dboo(db_crc32)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_crc32;
   }
-  if((a&DB_HAVAL) && (check_dboo(db_haval)!=RETFAIL)){
+  if((attr&DB_HAVAL) && (check_dboo(db_haval)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_haval;
   }
-  if((a&DB_GOST) && (check_dboo(db_gost)!=RETFAIL)){
+  if((attr&DB_GOST) && (check_dboo(db_gost)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_gost;
   }
-  if((a&DB_CRC32B) && (check_dboo(db_crc32b)!=RETFAIL)){
+  if((attr&DB_CRC32B) && (check_dboo(db_crc32b)!=RETFAIL)){
     conf->db_out_order[conf->db_out_size++]=db_crc32b;
   }
 #endif
+#ifdef WITH_ACL
+  if((attr&DB_ACL) && (check_dboo(db_acl)!=RETFAIL)){
+    conf->db_out_order[conf->db_out_size++]=db_acl;
+  }
+#endif
+  if((attr&DB_CHECKMASK) && (check_dboo(db_checkmask)!=RETFAIL)){
+    conf->db_out_order[conf->db_out_size++]=db_checkmask;
+  }
 }
 
 
@@ -258,7 +273,7 @@ void putbackvariable(char* var)
   }
   else {
     
-    error(230,"Variable %s not defined\n",tmp);
+    error(230,_("Variable %s not defined\n"),tmp);
 
     /*
      * We can use nondefined variable
@@ -308,7 +323,7 @@ int handle_endif(int doit,int allow_else){
   
   if(doit){
     int count=1;
-    error(230,"\nEating until @@endif\n");
+    error(230,_("\nEating until @@endif\n"));
     do {
       int i = conflex();
       switch (i) {
@@ -398,21 +413,26 @@ int do_ifxhost(int mode,char* name)
   s=(char*)malloc(sizeof(char)*MAXHOSTNAMELEN+1);
 
   if(gethostname(s,MAXHOSTNAMELEN)==-1){
-    error(0,"Couldn't get hostname %s",name);
+    error(0,_("Couldn't get hostname %s"),name);
+    free(s);
     return -1;
   }  
-  if(strcmp(name,s)==0)
+  if(strcmp(name,s)==0) {
     doit=1-doit;
-
+  }
+  free(s);
   return (handle_endif(doit,1));
 }
 
 list* append_rxlist(char* rx,int attr,list* rxlst)
 {
+  extern long conf_lineno; /* defined & set in conf_lex.l */
+    
   rx_rule* r=NULL;
   r=(rx_rule*)malloc(sizeof(rx_rule));
   r->rx=rx;
   r->attr=attr;
+  r->conf_lineno = conf_lineno;
   update_db_out_order(r->attr);
   rxlst=list_append(rxlst,(void*)r);
   
@@ -456,7 +476,7 @@ void do_dbindef(char* val)
      * both input and output urls */
     if(u==NULL||u->type==url_unknown||u->type==url_stdout
        ||u->type==url_stderr) {
-      error(0,"Unsupported input URL-type:%s\n",val);
+      error(0,_("Unsupported input URL-type:%s\n"),val);
     }
     else {
       conf->db_in_url=u;
@@ -470,21 +490,21 @@ void do_dboutdef(char* val)
 {
   url_t* u=NULL;
 
-  error(200,"Setting output database \"%s\"\n",val);
+  error(200,_("Setting output database \"%s\"\n"),val);
 
   if(conf->db_out_url==NULL){
     u=parse_url(val);
     /* FIXME Check the URL if you add support for databases that cannot be 
      * both input and output urls */
     if(u==NULL||u->type==url_unknown||u->type==url_stdin){
-      error(0,"Unsupported output URL-type:%s\n",val);
+      error(0,_("Unsupported output URL-type:%s\n"),val);
     }
     else{
       conf->db_out_url=u;
-      error(200,"Output database set to \"%s\" \"%s\"\n",val,u->value);
+      error(200,_("Output database set to \"%s\" \"%s\"\n"),val,u->value);
     }
   } else {
-    error(200,"Output database already set\n");
+    error(200,_("Output database already set\n"));
   }
 
   free(val);
@@ -499,7 +519,7 @@ void do_repurldef(char* val)
   /* FIXME Check the URL if you add support for databases that cannot be 
    * both input and output urls */
   if(u==NULL||u->type==url_unknown||u->type==url_stdin){
-    error(0,"Unsupported output URL-type:%s\n",val);
+    error(0,_("Unsupported output URL-type:%s\n"),val);
   } else {
     error_init(u);
   }
@@ -513,15 +533,15 @@ void do_verbdef(char* val)
   
   a=strtol(val,&err,10);
   if(*err!='\0' || a>255 || a<0 || errno==ERANGE){
-    error(0, "Illegal verbosity level:%s\n",val);
-    error(10,"Using previous value:%i\n",conf->verbose_level);
+    error(0, _("Illegal verbosity level:%s\n"),val);
+    error(10,_("Using previous value:%i\n"),conf->verbose_level);
     return;
   }    
   else {
     if(conf->verbose_level==-1){
       conf->verbose_level=a;
     }else {
-      error(210,"Verbosity already defined to %i\n",conf->verbose_level);
+      error(210,_("Verbosity already defined to %i\n"),conf->verbose_level);
     }
   }
 }
