@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/compare_db.c,v 1.34 2000/04/20 12:44:55 rammer Exp $
+ * $Header: /cvs-root-aide/aide/src/compare_db.c,v 1.44 2002/02/10 17:31:30 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,12 +34,28 @@
 #include "util.h"
 #include "commandconf.h"
 #include "gen_list.h"
+/*for locale support*/
+#include "locale-aide.h"
+/*for locale support*/
 
 #ifdef WITH_MHASH
 #include <mhash.h>
 #endif
 
-#include "cipher.h"
+#include <gcrypt.h>
+
+/*************/
+/* contruction area for report lines */
+const int old_col  = 12;   
+const int new_col  = 40;   
+
+const int part_len = 40; /* usable length of line[] */
+char      oline[40];
+char      nline[40];
+const char* entry_format="  %-9s: %-34s, %-34s\n";
+/*************/
+
+
 
 list* find_line_match(db_line* line,list* l)
 {
@@ -66,6 +82,41 @@ list* find_line_match(db_line* line,list* l)
   return NULL;
 }
 
+#ifdef WITH_ACL
+int compare_single_acl(aclent_t* a1,aclent_t* a2) {
+  if (a1->a_type!=a2->a_type ||
+      a1->a_id!=a2->a_id ||
+      a1->a_perm!=a2->a_perm) {
+    return RETFAIL;
+  }
+  return RETOK;
+}
+
+int compare_acl(acl_type* a1,acl_type* a2) {
+
+  int i;
+  if (a1==NULL && a2==NULL) {
+    return RETOK;
+  }
+  if (a1==NULL || a2==NULL) {
+    return RETFAIL;
+  }
+
+  if (a1->entries!=a2->entries) {
+    return RETFAIL;
+  }
+  /* Sort em up. */
+  aclsort(a1->entries,0,a1->acl);
+  aclsort(a2->entries,0,a2->acl);
+  for(i=0;i<a1->entries;i++){
+    if (compare_single_acl(a1->acl+i,a2->acl+i)==RETFAIL) {
+      return RETFAIL;
+    }
+  }
+  return RETOK;
+}
+#endif
+
 int compare_md_entries(byte* e1,byte* e2,int len)
 {
   if(e1!=NULL && e2!=NULL){
@@ -85,10 +136,8 @@ int compare_md_entries(byte* e1,byte* e2,int len)
   return RETFAIL;
 }
 
-
 int compare_lines(db_line* l1,db_line* l2,int ignorelist)
 {
-
   if (!(DB_LINKNAME&ignorelist)) {
     if(l1->linkname==NULL){
       if(l2->linkname!=NULL){
@@ -121,6 +170,8 @@ int compare_lines(db_line* l1,db_line* l2,int ignorelist)
     if(l1->perm!=l2->perm){
       return RETFAIL;
     }
+  } else {
+    error(0,"Ignoring permissions\n");
   }
   if (!(DB_UID&ignorelist)) {
     if(l1->uid!=l2->uid){
@@ -164,28 +215,28 @@ int compare_lines(db_line* l1,db_line* l2,int ignorelist)
   
   if (!(DB_MD5&ignorelist)) {  
     if(compare_md_entries(l1->md5,l2->md5,
-			  md_digest_length(DIGEST_ALGO_MD5))==RETFAIL){
+			  gcry_md_get_algo_dlen(GCRY_MD_MD5))==RETFAIL){
       return RETFAIL;
     }
   }
 
   if (!(DB_SHA1&ignorelist)) {
     if(compare_md_entries(l1->sha1,l2->sha1,
-			  md_digest_length(DIGEST_ALGO_SHA1))==RETFAIL){
+			  gcry_md_get_algo_dlen(GCRY_MD_SHA1))==RETFAIL){
       return RETFAIL;
     }
   }
   
   if (!(DB_RMD160&ignorelist)) {
     if(compare_md_entries(l1->rmd160,l2->rmd160,
-			  md_digest_length(DIGEST_ALGO_RMD160))==RETFAIL){
+			  gcry_md_get_algo_dlen(GCRY_MD_RMD160))==RETFAIL){
       return RETFAIL;
     }
   }
   
   if (!(DB_TIGER&ignorelist)) {
     if(compare_md_entries(l1->tiger,l2->tiger,
-			  md_digest_length(DIGEST_ALGO_TIGER))==RETFAIL){
+			  gcry_md_get_algo_dlen(GCRY_MD_TIGER))==RETFAIL){
       return RETFAIL;
     }
   }
@@ -218,7 +269,14 @@ int compare_lines(db_line* l1,db_line* l2,int ignorelist)
       return RETFAIL;
     }
   }
+#endif
 
+#ifdef WITH_ACL
+  if (!(DB_ACL&ignorelist)) {
+    if(compare_acl(l1->acl,l2->acl)==RETFAIL) {
+      return RETFAIL;
+    }
+  }
 #endif
   
   return RETOK;
@@ -226,45 +284,93 @@ int compare_lines(db_line* l1,db_line* l2,int ignorelist)
 
 void print_lname_changes(char*old,char*new)
 {
+  int ok = 0;
+
   if(old==NULL){
     if(new!=NULL){
-      error(5,"Linkname: old = <NULL> , new = %s\n",new);
+       snprintf(oline,part_len,"<NULL>");
+       snprintf(nline,part_len,"%s",new);
+       ok = 1;
+    }
+  } else if(new==NULL){
+       snprintf(oline,part_len,"%s",old);
+       snprintf(nline,part_len,"<NULL>");
+       ok = 1;
+   } else if(strcmp(old,new)!=0){
+        snprintf(oline,part_len,"%s",old);
+        snprintf(nline,part_len,"%s",new);
+        ok = 1;
+  }
+   if(ok)
+     error(5,(char*)entry_format,"Lname",oline,nline);
+}
+
+#ifdef WITH_ACL
+void print_single_acl(acl_type* acl){
+  char* aclt;
+  
+  if (acl==NULL) {
+    error(5,"<NULL>");
+  } else {
+    
+    aclt=acltotext(acl->acl,acl->entries);
+    if (aclt==NULL) {
+      error(5,"ERROR");
+    } else {
+      error(5,"%s ,",aclt);
+      free(aclt);
     }
-  }else if(new==NULL){
-    error(5,"Linkname: old = %s , new = <NULL>\n",old);
-  }else if(strcmp(old,new)!=0){
-    error(5,"Linkname: old = %s , new = %s\n",
-	  old,new);
   }
 }
 
-void print_md_changes(byte*old,byte*new,int len,char* fnam)
+void print_acl_changes(acl_type* old,acl_type* new) {
+  
+  if (compare_acl(old,new)==RETFAIL) {
+    error(5,"Acl: old = ");
+    print_single_acl(old);
+    error(5,"\n     new = ");
+    print_single_acl(new);
+  }
+  
+}
+#endif
+
+void print_md_changes(byte*old,byte*new,int len,char* name)
 {
+  int ok = 0;
   if(old!=NULL && new!=NULL){
     if(strncmp(old,new,len)!=0){
-      error(5,"%s: old = %s , new = %s\n",
-	    fnam,
-	    encode_base64(old,len),
-	    encode_base64(new,len));
+    	snprintf(oline,part_len,"%s",encode_base64(old,len));
+	snprintf(nline,part_len,"%s",encode_base64(new,len));
+	ok = 1;
     }
-  }else {
+  } else {
     if(old == NULL && new == NULL){
       return;
     }
-    error(5,"%s: old = ",fnam);
     if(old==NULL){
-      error(5,"0");
+      snprintf(oline,part_len,"NA");
     } else {
-      error(5,"%s",encode_base64(old,len));
+      snprintf(oline,part_len,"%s",encode_base64(old,len));
+      ok = 1;
     }
-    error(5," new = ");
+    /* OLD one */
     if(new==NULL){
-      error(5,"0");
+      snprintf(nline,part_len,"NA");
     }else {
-      error(5,"%s",encode_base64(new,len));
+      snprintf(nline,part_len,"%s",encode_base64(new,len));
+      ok = 1;
     }
-    error(5,"\n");
   }
+  if(ok)
+     error(5,(char*)entry_format,name,oline,nline);
+  }
+  
+int is_time_null(struct tm *ot)
+{
+    /* 1970-01-01 01:00:00 is year null */
+    return (ot->tm_year==70 && ot->tm_mon == 0 && ot->tm_mday == 1
+            && ot->tm_hour == 1 &&  ot->tm_min == 0 && ot->tm_sec == 0);
 }
 
 void print_time_changes(const char* name, time_t old_time, time_t new_time)
@@ -281,12 +387,53 @@ void print_time_changes(const char* name, time_t old_time, time_t new_time)
   
   nt = localtime(&(new_time));
   
-  error(5,"%s: old = %0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u,"
-	" new = %0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u\n", name,
+  if( is_time_null(ot) )
+      snprintf(oline,part_len,"NA");
+  else
+      snprintf(oline,part_len,
+               "%0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u",
            ot->tm_year+1900, ot->tm_mon+1, ot->tm_mday,
-	ot->tm_hour, ot->tm_min, ot->tm_sec,
+               ot->tm_hour, ot->tm_min, ot->tm_sec);
+  if( is_time_null(nt) )
+      snprintf(nline,part_len,"NA");
+  else
+      snprintf(nline,part_len,
+               "%0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u",
 	nt->tm_year+1900, nt->tm_mon+1, nt->tm_mday,
 	nt->tm_hour, nt->tm_min, nt->tm_sec);
+  error(5,(char*)entry_format,name,oline,nline); 
+}
+
+void print_int_changes(
+        const char* name,
+        int old,
+        int new
+        )
+{
+  snprintf(oline,part_len,"%i",old);
+  snprintf(nline,part_len,"%i",new);
+  error(5,(char*)entry_format,name,oline,nline); 
+}
+void print_ulong_changes(
+        const char* name,
+        unsigned long old,
+        unsigned long new
+        )
+{
+  snprintf(oline,part_len,"%lu",old);
+  snprintf(nline,part_len,"%lu",new);
+  error(5,(char*)entry_format,name,oline,nline);  
+}
+
+void print_string_changes(
+        const char* name,
+        const char* old,
+        const char* new
+        )
+{
+  snprintf(oline,part_len,"%s",old);
+  snprintf(nline,part_len,"%s",new);
+  error(5,(char*)entry_format,name,oline,nline); 
 }
 
 
@@ -295,31 +442,31 @@ void print_dbline_changes(db_line* old,db_line* new,int ignorelist)
   char* tmp=NULL;
   char* tmp2=NULL;
 
-  error(5,"\nFile: %s\n",old->filename);
-
+  if(S_ISDIR(new->perm_o)){
+    error(5,"\nDirectory: %s\n",old->filename);
+  }else {
+    error(5,"\nFile: %s\n",old->filename);
+  }
+  
   if(!(DB_LINKNAME&ignorelist)){
     print_lname_changes(old->linkname,new->linkname);
   }
-
   if (!(DB_SIZE&ignorelist)) {
     if(old->size!=new->size){
-      error(5,"Size: old = %u , new = %u\n",
-	    old->size,new->size);
+        print_ulong_changes("Size", old->size,new->size);
     }
   }
 
   if (!(DB_BCOUNT&ignorelist)) {
     if(old->bcount!=new->bcount){
-      error(5,"Bcount: old = %i , new = %i\n",
-	    old->bcount,new->bcount);
+          print_int_changes("Bcount", old->bcount,new->bcount);
     }
   }
   if (!(DB_PERM&ignorelist)) {
     if(old->perm!=new->perm){
       tmp=perm_to_char(old->perm);
       tmp2=perm_to_char(new->perm);
-      error(5,"Permissions: old = %s , new = %s\n",
-	    tmp,tmp2);
+      print_string_changes("Permissions", tmp,tmp2);
       free(tmp);
       free(tmp2);
       tmp=NULL;
@@ -329,15 +476,13 @@ void print_dbline_changes(db_line* old,db_line* new,int ignorelist)
   
   if (!(DB_UID&ignorelist)) {
     if(old->uid!=new->uid){
-      error(5,"Uid: old = %i , new = %i\n",
-	    old->uid,new->uid);
+        print_int_changes("Uid", old->uid,new->uid);
     }
   }
 
   if (!(DB_GID&ignorelist)) {
     if(old->gid!=new->gid){
-      error(5,"Gid: old = %i , new = %i\n",
-	    old->gid,new->gid);
+        print_int_changes("Gid", old->gid,new->gid);
     }
   }
   
@@ -361,38 +506,36 @@ void print_dbline_changes(db_line* old,db_line* new,int ignorelist)
 
   if (!(DB_INODE&ignorelist)) {
     if(old->inode!=new->inode){
-      error(5,"Inode: old = %i , new = %i\n",
-	    old->inode,new->inode);
+        print_int_changes("Inode", old->inode,new->inode);
     }
   }
   if (!(DB_LNKCOUNT&ignorelist)) {
     if(old->nlink!=new->nlink){
-      error(5,"Linkcount: old = %i , new = %i\n",
-	    old->nlink,new->nlink);
+        print_int_changes("Linkcount", old->nlink,new->nlink);
     }
   }
 
   if (!(DB_MD5&ignorelist)) {  
     print_md_changes(old->md5,new->md5,
-		     md_digest_length(DIGEST_ALGO_MD5),
+		     gcry_md_get_algo_dlen(GCRY_MD_MD5),
 		     "MD5");
   }
   
   if (!(DB_SHA1&ignorelist)) {
     print_md_changes(old->sha1,new->sha1,
-		     md_digest_length(DIGEST_ALGO_SHA1),
+		     gcry_md_get_algo_dlen(GCRY_MD_SHA1),
 		     "SHA1");
   }
 
   if (!(DB_RMD160&ignorelist)) {
     print_md_changes(old->rmd160,new->rmd160,
-		     md_digest_length(DIGEST_ALGO_RMD160),
+		     gcry_md_get_algo_dlen(GCRY_MD_RMD160),
 		     "RMD160");
   }
   
   if (!(DB_TIGER&ignorelist)) {
     print_md_changes(old->tiger,new->tiger,
-		     md_digest_length(DIGEST_ALGO_TIGER),
+		     gcry_md_get_algo_dlen(GCRY_MD_TIGER),
 		     "TIGER");
   }
   
@@ -422,20 +565,32 @@ void print_dbline_changes(db_line* old,db_line* new,int ignorelist)
   }
 #endif                   
 
+#ifdef WITH_ACL
+  if (!(DB_ACL&ignorelist)) {
+    print_acl_changes(old->acl,new->acl);
+  }
+#endif
+
 }
 
 void free_db_line(db_line* dl)
 {
-  free(dl->md5);
-  free(dl->sha1);
-  free(dl->rmd160);
-  free(dl->tiger);
-  free(dl->filename);
+
+#define checked_free(x) if(x) free(x)
+
+  checked_free(dl->md5);
+  checked_free(dl->sha1);
+  checked_free(dl->rmd160);
+  checked_free(dl->tiger);
+  checked_free(dl->filename);
+  checked_free(dl->linkname);
+  
+
 #ifdef WITH_MHASH
-  free(dl->crc32);
-  free(dl->crc32b);
-  free(dl->gost);
-  free(dl->haval);
+  checked_free(dl->crc32);
+  checked_free(dl->crc32b);
+  checked_free(dl->gost);
+  checked_free(dl->haval);
 
   dl->crc32=NULL;
   dl->crc32b=NULL;
@@ -446,6 +601,7 @@ void free_db_line(db_line* dl)
 
 
   dl->filename=NULL;
+  dl->linkname=NULL;
   dl->md5=NULL;
   dl->sha1=NULL;
   dl->rmd160=NULL;
@@ -455,9 +611,9 @@ void free_db_line(db_line* dl)
 
 void init_rxlst(list* rxlst)
 {
-  list* r=NULL;
-  rx_rule* rxrultmp=NULL;
-  regex_t* rxtmp=NULL;
+    list*    r         = NULL;
+    rx_rule* rxrultmp  = NULL;
+    regex_t* rxtmp     = NULL;
 
 
   for(r=rxlst;r;r=r->next){
@@ -469,7 +625,7 @@ void init_rxlst(list* rxlst)
     data=(char*)malloc(strlen(((rx_rule*)r->data)->rx)+1+1);
     
     if (data==NULL){
-      error(0,"Not enough memory for regexpr compile... exiting..\n");
+      error(0,_("Not enough memory for regexpr compile... exiting..\n"));
       abort();
     }
     
@@ -478,9 +634,10 @@ void init_rxlst(list* rxlst)
     data[0]='^';
     
     rxrultmp=((rx_rule*)r->data);
+    rxrultmp->conf_lineno=-1;
     rxtmp=(regex_t*)malloc(sizeof(regex_t));
     if( regcomp(rxtmp,data,REG_EXTENDED|REG_NOSUB)){
-      error(0,"Error in selective regexp:%s",((rx_rule*)r->data)->rx);
+      error(0,_("Error in selective regexp:%s"),((rx_rule*)r->data)->rx);
       free(data);
     }else {
       free(rxrultmp->rx);
@@ -530,7 +687,7 @@ void compare_db(list* new,db_config* conf)
 
   int ignorelist;
 
-  error(200,"compare_db()\n");
+  error(200,_("compare_db()\n"));
 
 
   /* With this we avoid unnecessary checking of removed files. */
@@ -564,14 +721,22 @@ void compare_db(list* new,db_config* conf)
 	  check_list_for_match(conf->equrxlst,old->filename,&tempignore)) &&
 	 !check_list_for_match(conf->negrxlst,old->filename,&tempignore)){
 	if(!(conf->action&DO_INIT)){
-	  error(5,"WARNING: Old db contains a file that shouldn\'t be there, run --init or --update\n");
+	  error(5,_("WARNING: Old db contains a file that shouldn\'t be there, run --init or --update\n"));
 	}
 	initdbwarningprinted=1;
       }
       removed=list_append(removed,(void*)old);
       nrem++;
     }else {
-      if(compare_lines(old,(db_line*)r->data,ignorelist)==RETFAIL){
+      int localignorelist=old->attr ^ ((db_line*)r->data)->attr;
+      
+      if (localignorelist!=0) {
+	error(21,"File %s in databases has different attributes, %i,%i\n",old->filename,old->attr,((db_line*)r->data)->attr);
+      }
+      
+      localignorelist|=ignorelist;
+      
+      if(compare_lines(old,(db_line*)r->data,localignorelist)==RETFAIL){
 	changednew=list_append(changednew,r->data);
 	changedold=list_append(changedold,(void*)old);
 	r->data=NULL;
@@ -604,27 +769,29 @@ void compare_db(list* new,db_config* conf)
   if(nadd!=0||nrem!=0||nchg!=0){
     struct tm* st=localtime(&(conf->start_time));
     
-    error(0,"AIDE found differences between database and filesystem!!\n");
-    error(5,"Start timestamp: %0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u\n",
+    error(0,_("AIDE found differences between database and filesystem!!\n"));
+    error(5,_("Start timestamp: %0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u\n"),
 	  st->tm_year+1900, st->tm_mon+1, st->tm_mday,
 	  st->tm_hour, st->tm_min, st->tm_sec);
-    error(0,"Summary:\nTotal number of files=%i,added files=%i"
-	  ",removed files=%i,changed files=%i\n\n",nfil,nadd,nrem,nchg);
-
+    error(0,_("Summary:\nTotal number of files=%i,added files=%i"
+	  ",removed files=%i,changed files=%i\n\n"),nfil,nadd,nrem,nchg);
+    
     if(nadd!=0){
-      error(5,"Added files:\n");
+      error(5,_("Added files:\n"));
       for(r=added;r;r=r->next){
 	error(5,"added:%s\n",((db_line*)r->data)->filename);
 	if(conf->verbose_level<20){
 	  if(S_ISDIR(((db_line*)r->data)->perm)){
-	    /*	    free_db_line((db_line*)r->data);
-	    free(r->data);
-	    r=list_delete_item(r);*/
+	    /*	    
+		    free_db_line((db_line*)r->data);
+		    free(r->data);
+		    r=list_delete_item(r);
+	    */
 	    eat_files_indir(r->next,((db_line*)r->data)->filename,&filesindir);
 	    if(filesindir>0){
 	      error(5,
-		    "added: THERE WERE ALSO %li "
-		    "FILES ADDED UNDER THIS DIRECTORY\n"
+		    _("added: THERE WERE ALSO %li "
+		    "FILES ADDED UNDER THIS DIRECTORY\n")
 		    ,filesindir);
 	    }
 	  }
@@ -634,29 +801,31 @@ void compare_db(list* new,db_config* conf)
     
 
     if(nrem!=0){
-      error(5,"Removed files:\n");
+      error(5,_("Removed files:\n"));
       for(r=removed;r;r=r->next){
 	error(5,"removed:%s\n",((db_line*)r->data)->filename);
       }
     }
 
     if(nchg!=0){
-      error(5,"Changed files:\n");
+      error(5,_("Changed files:\n"));
       for(r=changedold;r;r=r->next){
 	error(5,"changed:%s\n",((db_line*)r->data)->filename);
       }
     }
 
     if((conf->verbose_level>=5)&&(nchg!=0)){
-      error(5,"Detailed information about changes:\n");
+      error(5,_("Detailed information about changes:\n"));
       for(r=changedold,l=changednew;r;r=r->next,l=l->next){
+	int localignorelist=((db_line*)l->data)->attr^((db_line*)r->data)->attr;
+	localignorelist|=ignorelist;
 	print_dbline_changes((db_line*)r->data,
-			     (db_line*)l->data,ignorelist);
+			     (db_line*)l->data,localignorelist);
       }
     }
     conf->end_time=time(&(conf->end_time));
     st=localtime(&(conf->end_time));
-    error(20,"\nEnd timestamp: %0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u\n",
+    error(20,_("\nEnd timestamp: %0.4u-%0.2u-%0.2u %0.2u:%0.2u:%0.2u\n"),
 	  st->tm_year+1900, st->tm_mon+1, st->tm_mday,
 	  st->tm_hour, st->tm_min, st->tm_sec);
   }
