@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/db.c,v 1.33 2000/04/20 12:44:56 rammer Exp $
+ * $Header: /cvs-root-aide/aide/src/db.c,v 1.46 2001/11/26 07:13:42 pablo Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,7 +33,7 @@
 #include "report.h"
 #include "be.h"
 
-#include "cipher.h"
+#include <gcrypt.h>
 
 #ifdef WITH_MHASH
 #include <mhash.h>
@@ -42,6 +42,9 @@
 #include "base64.h"
 #include "util.h"
 #include "aide.h"
+/*for locale support*/
+#include "locale-aide.h"
+/*for locale support*/
 
 db_line* db_char2line(char** ss,db_config* conf);
 long readint(char* s,char* err);
@@ -50,29 +53,41 @@ long readoct(char* s,char* err);
 time_t base64totime_t(char*);
 int db_init(db_config* conf,int inout)
 {
+  void* rv=NULL;
   
   error(200,"db_init %i\n",inout);
-
+  
   if(inout==1){
-    conf->db_in=be_init(1,conf->db_in_url,0);
-    if(conf->db_in==NULL) {
+    
+    rv=be_init(1,conf->db_in_url,0);
+    if(rv==NULL) {
+      error(200,_("db_in is null\n"));      
       return RETFAIL;
     }
+    conf->db_in=rv;
+    error(200,_("db_in is nonnull\n"));
     return RETOK;
   }
   if(inout==0){
-
+    
 #ifdef WITH_ZLIB
-    conf->db_out=be_init(0,conf->db_out_url,conf->gzip_dbout);
-#else 
-    conf->db_out=be_init(0,conf->db_out_url,0);
+    if(conf->gzip_dbout){
+       rv=be_init(0,conf->db_out_url,conf->gzip_dbout);
+       conf->db_gzout=rv;
+    }
+    else{
+#endif
+      rv=be_init(0,conf->db_out_url,conf->gzip_dbout);
+      conf->db_out=rv;
+#ifdef WITH_ZLIB
+    }
 #endif
     
-    if(conf->db_out==NULL){
-      error(200,"db_out is null\n");
+    if(rv==NULL){
+      error(200,_("db_out is null\n"));
       return RETFAIL;
     }
-    error(200,"db_out is nonnull %s\n",conf->db_out_url->value);
+    error(200,_("db_out is nonnull %s\n"),conf->db_out_url->value);
     return RETOK;
   }
   return RETFAIL;
@@ -94,7 +109,8 @@ db_line* db_readline(db_config* conf){
 	s=db_char2line(ss,conf);
 
 	for(i=0;i<conf->db_in_size;i++){
-	  if(ss[conf->db_in_order[i]]!=NULL){
+	  if(conf->db_in_order[i]!=db_unknown && 
+	     ss[conf->db_in_order[i]]!=NULL){
 	    free(ss[conf->db_in_order[i]]);
 	    ss[conf->db_in_order[i]]=NULL;
 	  }
@@ -106,8 +122,16 @@ db_line* db_readline(db_config* conf){
     
     break;
   }
+#ifdef WITH_PSQL
+  case url_sql: {
+    error(255,"db_sql readline...");
+    s=db_readline_sql(conf);
+    
+    break;
+  }
+#endif
   default : {
-    error(0,"db_readline():Url-type backend not implemented\n");
+    error(0,_("db_readline():Url-type backend not implemented\n"));
     return NULL;
   }
   }
@@ -138,6 +162,9 @@ db_line* db_char2line(char** ss,db_config* conf){
   line->crc32b=NULL;
   line->haval=NULL;
   line->gost=NULL;
+#endif
+#ifdef WITH_ACL
+  line->acl=0;
 #endif
   line->perm=0;
   line->uid=0;
@@ -151,7 +178,8 @@ db_line* db_char2line(char** ss,db_config* conf){
   line->size=0;
   line->filename=NULL;
   line->linkname=NULL;
-  line->attr=0;
+  
+  line->attr=conf->attr; /* attributes from @@dbspec */
 
   for(i=0;i<conf->db_in_size;i++){
     switch (conf->db_in_order[i]) {
@@ -166,6 +194,7 @@ db_line* db_char2line(char** ss,db_config* conf){
       break;
     }
     case db_linkname : {
+      char *s = ss[conf->db_in_order[i]];
       if(ss[conf->db_in_order[i]]!=NULL){
 	if(ss[conf->db_in_order[i]][0]=='0'){
 	  if(ss[conf->db_in_order[i]][1]=='\0'){
@@ -175,13 +204,15 @@ db_line* db_char2line(char** ss,db_config* conf){
 	    line->linkname=strdup("");
 	    break;
 	  }else if(ss[conf->db_in_order[i]][1]=='0'){
-	    ss[conf->db_in_order[i]]++;
+	    memmove(s,s+1,strlen(s+1)+1); // Hope this removes core
+	    // dumpping in some environments. Has something to do with
+	    // memory (de)allocation.
 	  }
 	}
-	decode_string(ss[conf->db_in_order[i]]);
-	line->linkname=strdup(ss[conf->db_in_order[i]]);
+	decode_string(s);
+	line->linkname=strdup(s);
       } else {
-	error(0,"db_char2line():Error while reading database\n");
+	error(0,_("db_char2line():Error while reading database\n"));
 	abort();
       }
       break;
@@ -260,6 +291,34 @@ db_line* db_char2line(char** ss,db_config* conf){
 			       strlen(ss[conf->db_in_order[i]]));
       break;
     }
+#endif
+#ifdef WITH_ACL
+    case db_acl : {
+      char* endp,*pos;
+      int entries,lc;
+      line->acl=NULL;
+      
+      entries=strtol(ss[conf->db_in_order[i]],&endp,10);
+      if (endp==ss[conf->db_in_order[i]]) {
+ 	/* Something went wrong */
+	break;
+      }
+      pos=endp+1; /* Warning! if acl in database is corrupted then
+		     this will break down. */
+      
+      line->acl=malloc(sizeof(acl_type));
+      line->acl->entries=entries;
+      line->acl->acl=malloc(sizeof(aclent_t)*entries);
+      for (lc=0;lc<entries;lc++) {
+	line->acl->acl[lc].a_type=strtol(pos,&endp,10);
+	pos=endp+1;
+	line->acl->acl[lc].a_id=strtol(pos,&endp,10);
+	pos=endp+1;
+	line->acl->acl[lc].a_perm=strtol(pos,&endp,10);
+	pos=endp+1;
+      }
+      break;
+    }
 #endif
     case db_perm : {
       line->perm=readoct(ss[conf->db_in_order[i]],"permissions");
@@ -270,9 +329,19 @@ db_line* db_char2line(char** ss,db_config* conf){
       line->nlink=readint(ss[conf->db_in_order[i]],"nlink");
       break;
     }
-    
+
+    case db_attr : {
+      line->attr=readint(ss[conf->db_in_order[i]],"attr");
+      break;
+    }
+
+    case db_unknown : {
+      /* Unknown fields are ignored. */
+      break;
+    }
+  
     default : {
-      error(0,"Not implemented in db_char2line %i \n",conf->db_in_order[i]);
+      error(0,_("Not implemented in db_char2line %i \n"),conf->db_in_order[i]);
       return NULL;
     }
     
@@ -314,7 +383,7 @@ long readint(char* s,char* err){
   char* e;
   i=strtol(s,&e,10);
   if (e[0]!='\0') {
-    error(0,"Could not read %s from database",err);
+    error(0,_("Could not read %s from database"),err);
   }
   return i;
 }
@@ -324,7 +393,7 @@ long readoct(char* s,char* err){
   char* e;
   i=strtol(s,&e,8);
   if (e[0]!='\0') {
-    error(0,"Could not read %s from database. String %s \n",err,s);
+    error(0,_("Could not read %s from database. String %s \n"),err,s);
   }
   return i;
 }
@@ -337,7 +406,11 @@ int db_writespec(db_config* conf)
   case url_stderr:
   case url_fd:
   case url_file: {
-    if(conf->db_out!=NULL){
+    if(
+#ifdef WITH_ZLIB
+       (conf->gzip_dbout && conf->db_gzout) ||
+#endif
+       (conf->db_out!=NULL)){
       if(db_writespec_file(conf)==RETOK){
 	return RETOK;
       }
@@ -355,7 +428,7 @@ int db_writespec(db_config* conf)
   }
 #endif
   default:{
-    error(0,"Unknown output in db out.\n");    
+    error(0,_("Unknown output in db out.\n"));    
     return RETFAIL;
   }
   }
@@ -364,14 +437,18 @@ int db_writespec(db_config* conf)
 
 int db_writeline(db_line* line,db_config* conf){
 
-  if (line==NULL||conf==NULL) return 0;
+  if (line==NULL||conf==NULL) return RETOK;
   
   switch (conf->db_out_url->type) {
   case url_stdout:
   case url_stderr:
   case url_fd:
   case url_file: {
-    if (conf->db_out!=NULL) {
+    if (
+#ifdef WITH_ZLIB
+       (conf->gzip_dbout && conf->db_gzout) ||
+#endif
+       (conf->db_out!=NULL)) {
       if (db_writeline_file(line,conf)==RETOK) {
 	return RETOK;
       }
@@ -391,15 +468,51 @@ int db_writeline(db_line* line,db_config* conf){
   }
 #endif
   default : {
-    error(0,"Unknown output in db out.\n");    
+    error(0,_("Unknown output in db out.\n"));    
     return RETFAIL;
   } 
   }
   return RETFAIL;
 }
 
-/* Please FIXME */
 int db_close(db_config* conf)
 {
+  if (conf==NULL) return RETOK;
+  
+  switch (conf->db_out_url->type) {
+  case url_stdout:
+  case url_stderr:
+  case url_fd:
+  case url_file: {
+    if (
+#ifdef WITH_ZLIB
+       (conf->gzip_dbout && conf->db_gzout) ||
+#endif
+       (conf->db_out!=NULL)) {
+      if (db_close_file(conf)==RETOK) {
+	return RETOK;
+      }
+    }
+    return RETFAIL;
+    break;
+  }
+#ifdef WITH_PSQL
+  case url_sql: {
+    if (conf->db_out!=NULL) {
+      if (db_close_sql(conf->db_out)==RETOK) {
+	return RETOK;
+      } else {
+	return RETFAIL;
+      }
+    }
+    return RETOK;
+    break;
+  }
+#endif
+  default : {
+    error(0,_("db_close():Unknown output in db out.\n"));    
+    return RETFAIL;
+  } 
+  }
   return RETFAIL;
 }
