@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/db_file.c,v 1.45 2000/04/20 12:44:56 rammer Exp $
+ * $Header: /cvs-root-aide/aide/src/db_file.c,v 1.59 2002/02/10 19:10:32 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,7 +32,11 @@
 #include "conf_yacc.h"
 #include "util.h"
 #include "aide.h"
+#include "db_sql.h" /* typedefs */
 #include "commandconf.h"
+/*for locale support*/
+#include "locale-aide.h"
+/*for locale support*/
 
 #ifdef WITH_MHASH
 #include <mhash.h>
@@ -42,24 +46,14 @@
 #include <zlib.h>
 #endif
 
-#include "cipher.h"
+#include <gcrypt.h>
+
+#define BUFSIZE 16384
 
 extern int db_scan(); /* Rumaa.... */
 extern char* dbtext; /* Todella rumaa... */
 extern long db_lineno;
 
-char* db_names[] = {
-  "name", "lname" , "bcount", "perm", "uid", "gid", "size", "atime", "ctime",
-  "mtime", "inode", "count", "md5", "sha1", "rmd160", "tiger",
-  "crc32", "haval","gost","crc32b", "unknown" }; 
-
-const static int db_value[] = {
-  db_filename, db_linkname, db_bcount, db_perm, db_uid, db_gid,
-  db_size, db_atime, db_ctime, db_mtime, db_inode, db_lnkcount,
-  db_md5, db_sha1, db_rmd160, db_tiger, db_crc32, db_haval,
-  db_gost, db_crc32b,
-  db_unknown };
-
 
 extern void db_buff(FILE*);
 
@@ -96,28 +90,55 @@ void handle_gzipped_input(int out){
   /* NOT REACHED */
   return;
 }
-void handle_gzipped_output(int in) {
-  FILE*fh=NULL;
-  int*buf=malloc(ZBUFSIZE);
-  int nread=0;
-  
-  fh=fdopen(in,"rb");
+#endif
 
-  error(200,"handle_gzipped_ouput()\n");
-  while(!feof(fh)){
-    nread=fread(buf,1,ZBUFSIZE,fh);
-    gzwrite(conf->db_gzout,buf,nread);
 
+int dofflush(void)
+{
+
+  int retval;
+#ifdef WITH_ZLIB
+  if(conf->gzip_dbout){
+    retval=gzflush(conf->db_gzout,Z_SYNC_FLUSH);
+  }else {
+#endif
+    retval=fflush(conf->db_out); 
+#ifdef WITH_ZLIB
   }
-  gzclose(conf->db_gzout);
+#endif
 
-  error(200,"handle_gzipped_ouput(),exiting\n");
-  exit(0);
-  /* NOT REACHED */
-  return;
+  return retval;
 }
 
+int dofprintf( const char* s,...)
+{
+  int retval;
+  char* temp=NULL;
+  va_list ap;
+  
+  va_start(ap,s);
+#ifdef WITH_ZLIB
+  if(conf->gzip_dbout){
+    temp=(char*)malloc(BUFSIZE);
+    if(temp==NULL){
+      error(0,"Unable to alloc %i bytes\n",BUFSIZE);
+      return -1;
+    }
+    retval=vsnprintf(temp,BUFSIZE,s,ap);
+    retval=gzwrite(conf->db_gzout,temp,retval);
+    free(temp);
+  }else{
 #endif
+    retval=vfprintf(conf->db_out,s,ap);
+#ifdef WITH_ZLIB
+  }
+#endif
+  va_end(ap);
+
+  return retval;
+}
+
+
 
 int db_file_read_spec(db_config* conf){
   
@@ -156,6 +177,25 @@ int db_file_read_spec(db_config* conf){
 	  break;
 	}
       }
+      for (l=0;l<db_alias_size;l++){
+	
+	if (strcmp(db_namealias[l],dbtext)==0) {
+	  
+	  if (check_db_order(conf->db_in_order, conf->db_in_size,
+			     db_aliasvalue[l])==RETFAIL) {
+	    error(0,"Field %s redefined in @@dbspec\n",dbtext);
+	    conf->db_in_order[conf->db_in_size]=db_unknown;
+	  } else {
+	    conf->db_in_order[conf->db_in_size]=db_aliasvalue[l];
+	  }
+	  conf->db_in_size++;
+	  break;
+	}
+      }
+      if(l==db_unknown){
+	error(0,"Unknown field %s in database\n",dbtext);
+	conf->db_in_size++;
+      }
       break;
     }
     
@@ -171,10 +211,25 @@ int db_file_read_spec(db_config* conf){
     }
     }
   }
+
+  /* Lets generate attr from db_in_order if database does not have attr */
+  conf->attr=-1;
+
+  for (i=0;i<conf->db_in_size;i++) {
+    if (conf->db_in_order[i]==db_attr) {
+      conf->attr=1;
+    }
+  }
+  if (conf->attr==-1) {
+    conf->attr=0;
+    error(0,"Database does not have attr field.\nComparation may be incorrect\nGenerating attr-field from dbspec\nIt might be a good Idea to regenerate databases. Sorry.\n");
+    for(i=0;i<conf->db_in_size;i++) {
+      conf->attr|=1<<conf->db_in_order[i];
+    }
+  }
   return RETOK;
 }
 
-
 char** db_readline_file(db_config* conf){
   
   char** s=NULL;
@@ -400,19 +455,19 @@ int db_writechar(char* s,FILE* file,int i)
   int retval=0;
 
   if(i) {
-    fprintf(file," ");
+    dofprintf(" ");
   }
 
   if(s==NULL){
-    retval=fprintf(file,"0");
+    retval=dofprintf("0");
     return retval;
   }
   if(s[0]=='\0'){
-    retval=fprintf(file,"0-");
+    retval=dofprintf("0-");
     return retval;
   }
   if(s[0]=='0'){
-    retval=fprintf(file,"00");
+    retval=dofprintf("00");
     if(retval<0){
       return retval;
     }
@@ -420,7 +475,7 @@ int db_writechar(char* s,FILE* file,int i)
   }
 
   r=CLEANDUP(s);
-  retval=fprintf(file,"%s",r);
+  retval=dofprintf("%s",r);
   free(r);
 
   return retval;
@@ -429,10 +484,10 @@ int db_writechar(char* s,FILE* file,int i)
 int db_writeint(long i,FILE* file,int a)
 {
   if(a) {
-    fprintf(file," ");
+    dofprintf(" ");
   }
   
-  return fprintf(file,"%li",i);
+  return dofprintf("%li",i);
   
 }
 
@@ -443,15 +498,15 @@ int db_write_byte_base64(byte*data,size_t len,FILE* file,int i )
   
   tmpstr=encode_base64(data,len);
   if(i){
-    fprintf(file," ");
+    dofprintf(" ");
   }
 
   if(tmpstr){
-    retval=fprintf(file,tmpstr);
+    retval=dofprintf(tmpstr);
     free(tmpstr);
     return retval;
   }else {
-    return fprintf(file,"0");
+    return dofprintf("0");
   }
   return 0;
 
@@ -464,11 +519,11 @@ int db_write_time_base64(time_t i,FILE* file,int a)
   int retval=0;
 
   if(a){
-    fprintf(file," ");
+    dofprintf(" ");
   }
 
   if(i==0){
-    retval=fprintf(file,"0");
+    retval=dofprintf("0");
     return retval;
   }
 
@@ -484,7 +539,7 @@ int db_write_time_base64(time_t i,FILE* file,int a)
 
 
   tmpstr=encode_base64(ptr,strlen(ptr));
-  retval=fprintf(file,tmpstr);
+  retval=dofprintf(tmpstr);
   free(tmpstr);
   free(ptr);
 
@@ -495,10 +550,10 @@ int db_write_time_base64(time_t i,FILE* file,int a)
 int db_writeoct(long i, FILE* file,int a)
 {
   if(a) {
-    fprintf(file," ");
+    dofprintf(" ");
   }
   
-  return fprintf(file,"%lo",i);
+  return dofprintf("%lo",i);
   
 }
 
@@ -511,7 +566,7 @@ int db_writespec_file(db_config* conf)
   time_t tim=time(&tim);
   st=localtime(&tim);
 
-  retval=fprintf(conf->db_out,
+  retval=dofprintf(
 		 "# This file was generated by Aide, version %s\n"
 		 "# Time of generation was %.4u-%.2u-%.2u %.2u:%.2u:%.2u\n",
 		 AIDEVERSION,
@@ -522,14 +577,14 @@ int db_writespec_file(db_config* conf)
     return RETFAIL;
   }
 
-  retval=fprintf(conf->db_out,"@@db_spec ");
+  retval=dofprintf("@@db_spec ");
   if(retval==0){
     return RETFAIL;
   }
   for(i=0;i<conf->db_out_size;i++){
     for(j=0;j<db_unknown;j++){
       if(db_value[j]==conf->db_out_order[i]){
-	retval=fprintf(conf->db_out,"%s ",db_names[j]);
+	retval=dofprintf("%s ",db_names[j]);
 	if(retval==0){
 	  return RETFAIL;
 	}
@@ -537,13 +592,36 @@ int db_writespec_file(db_config* conf)
       }
     }
   }
-  retval=fprintf(conf->db_out,"\n");
+  retval=dofprintf("\n");
   if(retval==0){
     return RETFAIL;
   }
   return RETOK;
 }
 
+#ifdef WITH_SUN_ACL
+int db_writeacl(acl_type* acl,FILE* file,int a){
+  int i;
+
+  if(a) {
+    dofprintf(" ");
+  }
+  
+  if (acl==NULL) {
+    dofprintf("0");
+  } else {
+    
+    dofprintf("%i",acl->entries);
+    
+    for (i=0;i<acl->entries;i++) {
+      dofprintf(",%i,%i,%i", acl->acl[i].a_type, acl->acl[i].a_id,
+	      acl->acl[i].a_perm);
+    }
+  }
+  return RETOK;
+}
+#endif
+
 int db_writeline_file(db_line* line,db_config* conf){
   int i;
 
@@ -596,23 +674,23 @@ int db_writeline_file(db_line* line,db_config* conf){
     }
     case db_md5 : {
       db_write_byte_base64(line->md5,
-			   md_digest_length(DIGEST_ALGO_MD5),conf->db_out,i);
+			   gcry_md_get_algo_dlen(GCRY_MD_MD5),conf->db_out,i);
       break;
     }
     case db_sha1 : {
       db_write_byte_base64(line->sha1,
-			   md_digest_length(DIGEST_ALGO_SHA1),conf->db_out,i);
+			   gcry_md_get_algo_dlen(GCRY_MD_SHA1),conf->db_out,i);
       break;
     }
     case db_rmd160 : {
       db_write_byte_base64(line->rmd160,
-			   md_digest_length(DIGEST_ALGO_RMD160),
+			   gcry_md_get_algo_dlen(GCRY_MD_RMD160),
 			   conf->db_out,i);
       break;
     }
     case db_tiger : {
       db_write_byte_base64(line->tiger,
-			   md_digest_length(DIGEST_ALGO_TIGER),
+			   gcry_md_get_algo_dlen(GCRY_MD_TIGER),
 			   conf->db_out,i);
       break;
     }
@@ -646,6 +724,21 @@ int db_writeline_file(db_line* line,db_config* conf){
       break;
     }
 #endif
+    case db_attr : {
+      db_writeint(line->attr,
+		  conf->db_out,i);
+      break;
+    }
+#ifdef WITH_ACL
+    case db_acl : {
+      db_writeacl(line->acl,conf->db_out,i);
+      break;
+    }
+#endif
+    case db_checkmask : {
+      db_writeoct(line->attr,conf->db_out,i);
+      break;
+    }
     default : {
       error(0,"Not implemented in db_writeline_file %i\n",
 	    conf->db_out_order[i]);
@@ -656,6 +749,33 @@ int db_writeline_file(db_line* line,db_config* conf){
     
   }
 
-  fprintf(conf->db_out,"\n");
+  dofprintf("\n");
+  /* Can't use fflush because of zlib.*/
+  dofflush();
+
+  return RETOK;
+}
+
+int db_close_file(db_config* conf){
+
+#ifndef WITH_ZLIB
+  if(fclose(conf->db_out)){
+    error(0,"Unable to close database:%s\n",strerror(errno));
+    return RETFAIL;
+  }
+#else
+  if(conf->gzip_dbout){
+    if(gzclose(conf->db_gzout)){
+      error(0,"Unable to close gzdatabase:%s\n",strerror(errno));
+      return RETFAIL;
+    }
+  }else {
+    if(fclose(conf->db_out)){
+      error(0,"Unable to close database:%s\n",strerror(errno));
+      return RETFAIL;
+    }
+  }
+#endif
+
   return RETOK;
 }
