@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/do_md.c,v 1.35 2000/04/20 12:44:58 rammer Exp $
+ * $Header: /cvs-root-aide/aide/src/do_md.c,v 1.45 2001/11/26 07:13:42 pablo Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,11 +32,13 @@
 #include "do_md.h"
 #include "report.h"
 #include "list.h"
+/*for locale support*/
+#include "locale-aide.h"
+/*for locale support*/
 
 /* This define should be somewhere else */
 #define READ_BLOCK_SIZE 16384
 
-
 #ifdef WITH_MHASH
 #include <mhash.h>
 
@@ -45,7 +47,7 @@
 
 /* #define mhash_md_read(a,b) mhash_end(a[b*2]) */
 
-#define get_hash(a,b) if(dl->a){ \
+#define get_hash(a,b,c) if(dl->attr&c){ \
   dl->a=(byte*)malloc(mhash_get_block_size(b)); \
       memcpy(dl->a,mhash_end(mhash_mdh[b]), \
 	      mhash_get_block_size(b)); \
@@ -60,9 +62,13 @@
 #endif
 #endif
 
-#include "cipher.h"
-
+#include <gcrypt.h>
 
+void md_init_fail(const char* s,db_line* db,byte** hash,unsigned i) {
+  error(0,"Message diggest %s initialise failed\nDisabling %s for file %s\n",s,s,db->filename);
+  db->attr=db->attr&(~i);
+  (*hash)=0;
+}
 void free_hashes(db_line* dl){
 
   /* #define free_hash(a) if(dl->a!=NULL) {free(dl->a); dl->a=NULL;} */
@@ -92,14 +98,14 @@ list* do_md(list* file_lst,db_config* conf)
   ssize_t bread;
 #endif
 
-  MD_HANDLE mdh;
+  GCRY_MD_HD mdh;
   int fh=-1;
   void* buf=NULL;
   
   int i;
   off_t size;
   
-  mdh=md_open(0,0);
+  mdh=gcry_md_open(0,0);
 
 #ifndef HAVE_MMAP
 
@@ -114,10 +120,11 @@ list* do_md(list* file_lst,db_config* conf)
   
   for(l=file_lst;l;l=l->next){
     dl=(db_line*)l->data;
-    if(!(dl->md5 || dl->sha1 || dl->tiger || dl->rmd160 
+    if(!(dl->attr&DB_MD5 || dl->attr&DB_SHA1 || dl->attr&DB_TIGER ||
+	 dl->attr&DB_RMD160
 #ifdef WITH_MHASH
-	 || dl->crc32 || dl->haval || 
-	 dl->gost || dl->crc32b 
+	 || dl->attr&DB_CRC32 || dl->attr&DB_HAVAL || 
+	 dl->attr&DB_GOST || dl->attr&DB_CRC32B
 #endif
 	 ))
       continue;
@@ -131,43 +138,74 @@ list* do_md(list* file_lst,db_config* conf)
     
     /* We use the internal algorithms for md5,sha1,tiger,rmd160 */
     
-    if(dl->md5){
-      md_enable(mdh,DIGEST_ALGO_MD5);
+    if(dl->attr&DB_MD5){
+      if (gcry_md_enable(mdh,GCRY_MD_MD5)) {
+	md_init_fail("md5",dl,&(dl->md5),DB_MD5);
+	};
     }
-    if(dl->sha1){
-      md_enable(mdh,DIGEST_ALGO_SHA1);
+    if(dl->attr&DB_SHA1){
+      if (gcry_md_enable(mdh,GCRY_MD_SHA1)) {
+	md_init_fail("sha1",dl,&(dl->sha1),DB_SHA1);	
+      }
     }
-    if(dl->tiger){
-      md_enable(mdh,DIGEST_ALGO_TIGER);
+    if(dl->attr&DB_TIGER){
+      if (gcry_md_enable(mdh,GCRY_MD_TIGER)) {
+	md_init_fail("tiger",dl,&(dl->tiger),DB_TIGER);
+      }
     }
-    if(dl->rmd160){
-      md_enable(mdh,DIGEST_ALGO_RMD160);
+
+#define RMD160_LIBGCRYPT
+    
+    if (dl->attr&DB_RMD160) {
+      if
+#ifdef RMD160_LIBGCRYPT
+  (gcry_md_enable(mdh,GCRY_MD_RMD160))
+#endif /* RMD160_LIBGCRYPT */
+#ifdef RMD160_MHASH
+    ((mhash_mdh[MHASH_RIPEMD160]=mhash_init(MHASH_RIPEMD160))==MHASH_FAILED)
+#endif /* RMD160_MHASH */
+#ifdef NO_RMD160
+    (0==0) /* :) */
+#endif /* NO_RMD160  */
+	{
+	  md_init_fail("rmd160",dl,&(dl->rmd160),DB_RMD160);
+	}
     }
 #ifdef WITH_MHASH
-    if(dl->crc32){
-      if((mhash_mdh[MHASH_CRC32]=mhash_init(MHASH_CRC32))==MHASH_FAILED){
+    if(dl->attr&DB_CRC32){
+
+      if
+	
+	((mhash_mdh[MHASH_CRC32]=mhash_init(MHASH_CRC32))==MHASH_FAILED)
+	
+	{
 	error(0,"mhash_init() failed for CRC32\n");
+	md_init_fail("CRC32",dl,&(dl->crc32),DB_CRC32);	
       }
+
     }
-    if(dl->haval){
+    if(dl->attr&DB_HAVAL){
       if((mhash_mdh[MHASH_HAVAL256]=mhash_init(MHASH_HAVAL256))==MHASH_FAILED){
 	error(0,"mhash_init() failed for HAVAL256\n");
+	md_init_fail("HAVAL",dl,&(dl->haval),DB_HAVAL);	
       }
     }
-    if(dl->gost){
+    if(dl->attr&DB_GOST){
       if((mhash_mdh[MHASH_GOST]=mhash_init(MHASH_GOST))==MHASH_FAILED){
 	error(0,"mhash_init() failed for GOST\n");
+	md_init_fail("GOST",dl,&(dl->gost),DB_GOST);	
       }
     }
-    if(dl->crc32b){
+    if(dl->attr&DB_CRC32B){
       if((mhash_mdh[MHASH_CRC32B]=mhash_init(MHASH_CRC32B))==MHASH_FAILED){
 	error(0,"mhash_init() failed for CRC32B\n");
+	md_init_fail("CRC32B",dl,&(dl->crc32b),DB_CRC32B);	
       }
     }
 #endif
     
     
-    error(220,"Opening %s for md check\n",dl->filename);
+    error(220,_("Opening %s for md check\n"),dl->filename);
     if(dl->size > 0) {
       if(!(fh=open(dl->filename,O_RDONLY))){
 	error(5,"do_md():open() for %s failed:%s\n",
@@ -181,7 +219,7 @@ list* do_md(list* file_lst,db_config* conf)
 	error(20,"File size has changed for %s while aide was running (%d->%d).\n",dl->filename,dl->size,size);
 
 	dl->size=size;
-	if(dl->size==0){
+	if(dl->size<=0){
 	  goto dummywrite;
 	}
       }
@@ -222,7 +260,7 @@ list* do_md(list* file_lst,db_config* conf)
 	/* If the file is truncated after lseek */
 	/* Aide could cause SIGBUS and die */
 	/* An error message is displayed in this case */
-      md_write(mdh,(byte*)buf,dl->size);	
+      gcry_md_write(mdh,(byte*)buf,dl->size);	
 #ifdef WITH_MHASH
 	for(i=0;i<mhash_count();i++) {
 	  if (mhash_mdh[i]!=MHASH_FAILED) {
@@ -236,12 +274,12 @@ list* do_md(list* file_lst,db_config* conf)
 #endif /* HAVE_MMAP */
 #ifndef HAVE_MMAP
       /* We must go back to the beginning for read */
-      lseek(fh,0,SEEK_SET);
+      if(lseek(fh,0,SEEK_SET)>0);
     /* Let's read */
 	do{
 	  bread=read(fh,buf,READ_BLOCK_SIZE);
 	  
-	  md_write(mdh,(byte*)buf,bread);
+	  gcry_md_write(mdh,(byte*)buf,bread);
 #ifdef WITH_MHASH
 	  for(i=0;i<mhash_count();i++) {
 	    if (mhash_mdh[i]!=MHASH_FAILED) {
@@ -258,7 +296,7 @@ list* do_md(list* file_lst,db_config* conf)
       }
     } else {
     dummywrite:
-      md_write(mdh,(byte*)buf,0);
+      gcry_md_write(mdh,(byte*)buf,0);
 #ifdef WITH_MHASH
 	for(i=0;i<mhash_count();i++) {
 	  if (mhash_mdh[i]!=MHASH_FAILED) {
@@ -267,40 +305,40 @@ list* do_md(list* file_lst,db_config* conf)
 	} 
 #endif
     }
-    md_final(mdh); /* Let's flush the buffers */
+    gcry_md_final(mdh); /* Let's flush the buffers */
 
-    if(dl->md5){
-      dl->md5=(byte*)malloc(md_digest_length(DIGEST_ALGO_MD5));
-      memcpy(dl->md5,md_read(mdh,DIGEST_ALGO_MD5),
-              md_digest_length(DIGEST_ALGO_MD5));
+    if(dl->attr&DB_MD5){
+      dl->md5=(byte*)malloc(gcry_md_get_algo_dlen(GCRY_MD_MD5));
+      memcpy(dl->md5,gcry_md_read(mdh,GCRY_MD_MD5),
+              gcry_md_get_algo_dlen(GCRY_MD_MD5));
     }
 
-    if(dl->sha1){
-      dl->sha1=(byte*)malloc(md_digest_length(DIGEST_ALGO_SHA1));
-      memcpy(dl->sha1,md_read(mdh,DIGEST_ALGO_SHA1),
-	      md_digest_length(DIGEST_ALGO_SHA1));
+    if(dl->attr&DB_SHA1){
+      dl->sha1=(byte*)malloc(gcry_md_get_algo_dlen(GCRY_MD_SHA1));
+      memcpy(dl->sha1,gcry_md_read(mdh,GCRY_MD_SHA1),
+	      gcry_md_get_algo_dlen(GCRY_MD_SHA1));
     }
-    if(dl->tiger){
-      dl->tiger=(byte*)malloc(md_digest_length(DIGEST_ALGO_TIGER));
-      memcpy(dl->tiger,md_read(mdh,DIGEST_ALGO_TIGER),
-	      md_digest_length(DIGEST_ALGO_TIGER));
+    if(dl->attr&DB_TIGER){
+      dl->tiger=(byte*)malloc(gcry_md_get_algo_dlen(GCRY_MD_TIGER));
+      memcpy(dl->tiger,gcry_md_read(mdh,GCRY_MD_TIGER),
+	      gcry_md_get_algo_dlen(GCRY_MD_TIGER));
     }
-    if(dl->rmd160){
-      dl->rmd160=(byte*)malloc(md_digest_length(DIGEST_ALGO_RMD160));
-      memcpy(dl->rmd160,md_read(mdh,DIGEST_ALGO_RMD160),
-	      md_digest_length(DIGEST_ALGO_RMD160));
+    if(dl->attr&DB_RMD160){
+      dl->rmd160=(byte*)malloc(gcry_md_get_algo_dlen(GCRY_MD_RMD160));
+      memcpy(dl->rmd160,gcry_md_read(mdh,GCRY_MD_RMD160),
+	      gcry_md_get_algo_dlen(GCRY_MD_RMD160));
     }
 
-    md_reset(mdh);
+    gcry_md_reset(mdh);
 #ifdef WITH_MHASH
-    get_hash(crc32,MHASH_CRC32);
-    get_hash(haval,MHASH_HAVAL256);
-    get_hash(gost,MHASH_GOST);
-    get_hash(crc32b,MHASH_CRC32B);
+    get_hash(crc32,MHASH_CRC32,DB_CRC32);
+    get_hash(haval,MHASH_HAVAL256,DB_HAVAL);
+    get_hash(gost,MHASH_GOST,DB_GOST);
+    get_hash(crc32b,MHASH_CRC32B,DB_CRC32B);
 #endif
     }
   
-  md_close(mdh);
+  gcry_md_close(mdh);
 
 #ifndef HAVE_MMAP
 
