@@ -1,7 +1,8 @@
-/* aide, Advanced Intrusion Detection Environment
+/* aide, Advanced Intrusion Detection Envir
+onment
  *
  * Copyright (C) 1999,2000 Rami Lehti and Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/error.c,v 1.16 2000/04/20 12:44:58 rammer Exp $
+ * $Header: /cvs-root-aide/aide/src/error.c,v 1.19 2002/02/10 17:49:04 pablo Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,15 +19,24 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "../config.h"
+
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
 
+#ifdef HAVE_SYSLOG
+#include <syslog.h>
+#endif
+
 #include "report.h"
 #include "list.h"
 #include "aide.h"
 #include "be.h"
+/*for locale support*/
+#include "locale-aide.h"
+/*for locale support*/
 
 int cmp_url(url_t* url1,url_t* url2){
   
@@ -40,18 +50,32 @@ int error_init(url_t* url)
   FILE* fh=NULL;
   
   if(conf->verbose_level>=200){
-    error(5,"WARNING: Debug output enabled\n");
+    error(5,_("WARNING: Debug output enabled\n"));
   }
 
   for(r=conf->report_url;r;r=r->next){
     
     if (cmp_url((url_t*)r->data,url)) {
       
-      error(5,"WARNING: Already have report output %s\n",url->value);
+      error(5,_("WARNING: Already have report output %s\n"),url->value);
       return RETOK;
     }
     
   }
+
+
+  if (url->type==url_syslog) {
+    conf->report_syslog++;
+#ifdef HAVE_SYSLOG
+    openlog(AIDE_IDENT,AIDE_LOGOPT, AIDE_SYSLOG_FACILITY);
+
+    return RETOK;
+#endif
+#ifndef HAVE_SYSLOG
+    error(0,_("This binary has no syslog support\n"));
+    return RETFAIL;
+#endif
+  }
   
   fh=be_init(0,url,0);
   if(fh!=NULL) {
@@ -60,7 +84,7 @@ int error_init(url_t* url)
     return RETOK;
   }
   
-  error(0,"Cannot open %s for writing\n",url->value);
+  error(0,_("Cannot open %s for writing\n"),url->value);
 
   return RETFAIL;
 
@@ -81,6 +105,15 @@ void error(int errorlevel,char* error_msg,...)
       return;
     }
   }
+  
+#ifdef HAVE_SYSLOG
+  if (conf->report_syslog!=0) {
+    va_start(ap,error_msg);
+    syslog(SYSLOG_PRIORITY,error_msg,ap);
+    va_end(ap);
+  }
+#endif
+  
 
   if(conf->report_fd==NULL){
     /* We haven't initialized yet so let's use the default */
@@ -106,3 +139,4 @@ void error(int errorlevel,char* error_msg,...)
 
   return;
 }
+
