@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000 Rami Lehti,Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/gen_list.c,v 1.62 2000/04/20 12:44:58 rammer Exp $
+ * $Header: /cvs-root-aide/aide/src/gen_list.c,v 1.82 2002/02/11 09:27:23 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,6 +27,7 @@
 #include <limits.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <ustat.h>
 #include <errno.h>
 #include <time.h>
 #include "report.h"
@@ -37,15 +38,19 @@
 #include "seltree.h"
 #include "db_config.h"
 #include "report.h"
+/*for locale support*/
+#include "locale-aide.h"
+/*for locale support*/
 
 #define CLOCK_SKEW 5
 
 #ifdef WITH_MHASH
 #include <mhash.h>
-#else 
-#include "cipher.h"
 #endif
 
+#include <gcrypt.h>
+
+
 char* strrxtok(char* rx)
 {
   char*p=NULL;
@@ -65,6 +70,8 @@ char* strrxtok(char* rx)
       case '^':
       case '$':
       case '*':
+      case '.':
+      case '[':
 	/* FIXME: The '\\' character should be handled more gracefully. */
 	/* That is, if it is the only special character then */
 	/* The next character should be taken literally so */
@@ -127,7 +134,23 @@ seltree* get_seltree_node(seltree* tree,char* path)
   return NULL;
 }
 
-seltree* new_seltree_node(seltree* tree,char*path,int isrx)
+void copy_rule_ref(seltree* node, rx_rule* r)
+{
+    if( r!=NULL ){
+        node->conf_lineno = r->conf_lineno;  
+        node->rx=(char*)malloc(strlen(r->rx)+1);
+        strcpy(node->rx,r->rx);
+    } else {
+        node->conf_lineno = -1;
+        node->rx=NULL;
+    }
+}
+
+seltree* new_seltree_node(
+        seltree* tree,
+        char*path,
+        int isrx,
+        rx_rule*r)
 {
   seltree* node=NULL;
   seltree* parent=NULL;
@@ -139,6 +162,9 @@ seltree* new_seltree_node(seltree* tree,char*path,int isrx)
   node->neg_rx_lst=NULL;
   node->equ_rx_lst=NULL;
   node->checked=0;
+
+  copy_rule_ref(node,r);
+
   if(tree!=NULL){
     if(isrx){
       parent=get_seltree_node(tree,strrxtok(path));
@@ -147,9 +173,9 @@ seltree* new_seltree_node(seltree* tree,char*path,int isrx)
     }      
     if(parent==NULL){
       if(isrx){
-	parent=new_seltree_node(tree,strrxtok(path),isrx);
+	parent=new_seltree_node(tree,strrxtok(path),isrx,r);
       }else {
-	parent=new_seltree_node(tree,strlastslash(path),isrx);
+	parent=new_seltree_node(tree,strlastslash(path),isrx,r);
       }
     }
     
@@ -161,49 +187,55 @@ seltree* new_seltree_node(seltree* tree,char*path,int isrx)
   return node;
 }
 
-void gen_seltree(list* rxlist,seltree* tree,char type){
-
-  regex_t* rxtmp=NULL;
-  seltree* curnode=NULL;
-  list* r=NULL;
-  char* rxtok=NULL;
-  rx_rule* rxc=NULL;
+void gen_seltree(list* rxlist,seltree* tree,char type)
+{
+  regex_t*     rxtmp   = NULL;
+  seltree*     curnode = NULL;
+  list*        r       = NULL;
+  char*        rxtok   = NULL;
+  rx_rule*     rxc     = NULL;
 
   for(r=rxlist;r;r=r->next){
     char* data;
+    rx_rule* curr_rule = (rx_rule*)r->data;
     
-    rxtok=strrxtok(((rx_rule*)r->data)->rx);
+    
+    rxtok=strrxtok(curr_rule->rx);
     curnode=get_seltree_node(tree,rxtok);
     if(curnode==NULL){
-      curnode=new_seltree_node(tree,rxtok,1);
+      curnode=new_seltree_node(tree,rxtok,1,curr_rule);
     }
     
     /* We have to add '^' to the first charaster of string... 
      *
      */
 
-    data=(char*)malloc(strlen(((rx_rule*)r->data)->rx)+1+1);
+    data=(char*)malloc(strlen(curr_rule->rx)+1+1);
 
     if (data==NULL){
-      error(0,"Not enough memory for regexpr compile... exiting..\n");
+      error(0,_("Not enough memory for regexpr compile... exiting..\n"));
       abort();
     }
     
     /* FIX ME! (returnvalue) */
     
-    strcpy(data+1,((rx_rule*)r->data)->rx);
+    strcpy(data+1,curr_rule->rx);
     
     data[0]='^';
     
     rxtmp=(regex_t*)malloc(sizeof(regex_t));
     if( regcomp(rxtmp,data,REG_EXTENDED|REG_NOSUB)){
-      error(0,"Error in selective regexp:%s",((rx_rule*)r->data)->rx);
+      error(0,_("Error in selective regexp:%s"),curr_rule->rx);
       free(data);
     }else{
+      /* replace regexp text with regexp compiled */
       rxc=(rx_rule*)malloc(sizeof(rx_rule));
       rxc->rx=data;
+      /* and copy the rest */
       rxc->crx=rxtmp;
-      rxc->attr=((rx_rule*)r->data)->attr;
+      rxc->attr=curr_rule->attr;
+      rxc->conf_lineno=curr_rule->conf_lineno;
+
       switch (type){
       case 's':{
 	curnode->sel_rx_lst=list_append(curnode->sel_rx_lst,(void*)rxc);
@@ -231,19 +263,19 @@ list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
 {
   db_line* fil=NULL;
   time_t cur_time;
-  struct stat fs;
+  struct AIDE_STAT_TYPE fs;
   int sres=0;
   
-  error(220, "Adding %s to filelist\n",filename);
+  error(220, _("Adding %s to filelist\n"),filename);
   
   fil=(db_line*)malloc(sizeof(db_line));
-  fil->attr=attr;
+  fil->attr=attr|DB_FILENAME|DB_LINKNAME;
   fil->filename=(char*)malloc(sizeof(char)*strlen(filename)+1);
   strncpy(fil->filename,filename,strlen(filename));
   fil->filename[strlen(filename)]='\0';
   /* We want to use lstat here instead of stat since we want *
    * symlinks stats not the file that it points to. */
-  sres=lstat(fil->filename,&fs);
+  sres=AIDE_LSTAT_FUNC(fil->filename,&fs);
   if(sres==-1){
     char* er=strerror(errno);
     if (er==NULL) {
@@ -262,32 +294,61 @@ list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
   if (cur_time==(time_t)-1) {
     char* er=strerror(errno);
     if (er==NULL) {
-      error(0,"Can not get current time. strerror failed for %i\n",errno);
+      error(0,_("Can not get current time. strerror failed for %i\n"),errno);
     } else {
-      error(0,"Can not get current time with reason %s\n",er);
+      error(0,_("Can not get current time with reason %s\n"),er);
     }
   } else {
     
     if(fs.st_atime>cur_time){
-      error(CLOCK_SKEW,"%s atime in future\n",fil->filename);
+      error(CLOCK_SKEW,_("%s atime in future\n"),fil->filename);
     }
     if(fs.st_mtime>cur_time){
-      error(CLOCK_SKEW,"%s mtime in future\n",fil->filename);
+      error(CLOCK_SKEW,_("%s mtime in future\n"),fil->filename);
     }
     if(fs.st_ctime>cur_time){
-      error(CLOCK_SKEW,"%s ctime in future\n",fil->filename);
+      error(CLOCK_SKEW,_("%s ctime in future\n"),fil->filename);
     }
   }
+  
+  /*
+    Here we should check if we need to add it..
+   */  
+  
+  {
+    struct ustat buf;
+    if (ustat(fs.st_dev, &buf) != 0 || buf.f_fname[0]==0) {
+      
+    } else {
+      
+    }
 
+  }
+  
   
   fil->perm_o=fs.st_mode;
   fil->size_o=fs.st_size;
 
-  if(S_ISLNK(fs.st_mode)){
+  if((S_ISLNK(fs.st_mode))){
     char* lnktmp=NULL;
     char* lnkbuf=NULL;
     int len=0;
+    
+#ifdef WITH_ACL   
+    if(conf->no_acl_on_symlinks!=-1) {
+      fil->attr&=(~DB_ACL);
+    }
+#endif   
 
+    if(conf->warn_dead_symlinks!=-1) {
+      struct AIDE_STAT_TYPE fs;
+      int sres;
+      sres=AIDE_STAT_FUNC(fil->filename,&fs);
+      if (sres!=0) {
+	error(5,"Dead symlink detected at %s\n",fil->filename);
+      }
+    }
+    
     if(conf->symlinks_found==0){
       int it=0;
       DB_FIELD dbtmp;
@@ -305,14 +366,14 @@ list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
 
     lnktmp=(char*)malloc(_POSIX_PATH_MAX+1);
     if(lnktmp==NULL){
-      error(0,"malloc failed in add_file_to_list()\n");
+      error(0,_("malloc failed in add_file_to_list()\n"));
       abort();
     }
  
     len=readlink(fil->filename,lnktmp,_POSIX_PATH_MAX+1);
     lnkbuf=(char*)malloc(len+1);
     if(lnkbuf==NULL){
-      error(0,"malloc failed in add_file_to_list()\n");
+      error(0,_("malloc failed in add_file_to_list()\n"));
       abort();
     }
 
@@ -320,79 +381,118 @@ list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
     lnkbuf[len]='\0';
     fil->linkname=lnkbuf;
     free(lnktmp);
+    fil->attr|=DB_LINKNAME;
   }else {
     fil->linkname=NULL;
+    /* Just remove linkname avaivilibity bit from this entry */
+    fil->attr&=(~DB_LINKNAME); 
   }
 
   
-  if(DB_INODE&attr){
+  if(DB_INODE&fil->attr){
     fil->inode=fs.st_ino;
   } else {
     fil->inode=0;
   }
 
-  if(DB_UID&attr) {
+  if(DB_UID&fil->attr) {
     fil->uid=fs.st_uid;
   }else {
     fil->uid=0;
   }
 
-  if(DB_GID&attr){
+  if(DB_GID&fil->attr){
     fil->gid=fs.st_gid;
   }else{
     fil->gid=0;
   }
 
-  if(DB_PERM&attr){
+  if(DB_PERM&fil->attr){
     fil->perm=fs.st_mode;
   }else{
     fil->perm=0;
   }
 
-  if(DB_SIZE&attr||DB_SIZEG&attr){
+  if(DB_SIZE&fil->attr||DB_SIZEG&fil->attr){
     fil->size=fs.st_size;
   }else{
     fil->size=0;
   }
   
-  if(DB_LNKCOUNT&attr){
+  if(DB_LNKCOUNT&fil->attr){
     fil->nlink=fs.st_nlink;
   }else {
     fil->nlink=0;
   }
 
-  if(DB_MTIME&attr){
+  if(DB_MTIME&fil->attr){
     fil->mtime=fs.st_mtime;
   }else{
     fil->mtime=0;
   }
 
-  if(DB_CTIME&attr){
+  if(DB_CTIME&fil->attr){
     fil->ctime=fs.st_ctime;
   }else{
     fil->ctime=0;
   }
   
-  if(DB_ATIME&attr){
+  if(DB_ATIME&fil->attr){
     fil->atime=fs.st_atime;
   }else{
     fil->atime=0;
   }
 
-  if(DB_BCOUNT&attr){
+  if(DB_BCOUNT&fil->attr){
     fil->bcount=fs.st_blocks;
   } else {
     fil->bcount=0;
   }
 
+#ifdef WITH_ACL
+  if(DB_ACL&fil->attr) { /* There might be a bug here. */
+    int res;
+    fil->acl=malloc(sizeof(acl_type));
+    fil->acl->entries=acl(fil->filename,GETACLCNT,0,NULL);
+    if (fil->acl->entries==-1) {
+      char* er=strerror(errno);
+      fil->acl->entries=0;
+      if (er==NULL) {
+	error(0,"ACL query failed for %s. strerror failed for %i\n",fil->filename,errno);
+      } else {
+	error(0,"ACL query failed for %s:%s\n",fil->filename,er);
+      }
+    } else {
+      fil->acl->acl=malloc(sizeof(aclent_t)*fil->acl->entries);
+      res=acl(fil->filename,GETACL,fil->acl->entries,fil->acl->acl);
+      if (res==-1) {
+	error(0,"ACL error %s\n",strerror(errno));
+      } else {
+	if (res!=fil->acl->entries) {
+	  error(0,"Tried to read %i acl but got %i\n",fil->acl->entries,res);
+	}
+      }
+    }
+  }else{
+    fil->acl=NULL;
+  }
+
+#endif
+  
   if(S_ISDIR(fs.st_mode)||S_ISCHR(fs.st_mode)
      ||S_ISBLK(fs.st_mode)||S_ISFIFO(fs.st_mode)
      ||S_ISLNK(fs.st_mode)||S_ISSOCK(fs.st_mode)){
+    
+    fil->attr&=(~DB_MD5)&(~DB_SHA1)&(~DB_RMD160)&(~DB_TIGER);
+    
     fil->md5=NULL;
     fil->sha1=NULL;
     fil->tiger=NULL;
     fil->rmd160=NULL;
 #ifdef WITH_MHASH
+
+    fil->attr&=(~DB_CRC32)&(~DB_HAVAL)&(~DB_CRC32B);
+
     fil->crc32=NULL;
     fil->crc32b=NULL;
     fil->haval=NULL;
@@ -401,15 +501,15 @@ list* add_file_to_list(list* listp,char*filename,int attr,int* addok)
   }else {
     /* 1 if needs to be set
      * 0 otherwise */
-    fil->md5=DB_MD5&attr?(byte*)"":NULL;
-    fil->sha1=DB_SHA1&attr?(byte*)"":NULL;
-    fil->tiger=DB_TIGER&attr?(byte*)"":NULL;
-    fil->rmd160=DB_RMD160&attr?(byte*)"":NULL;
+    fil->md5=DB_MD5&fil->attr?(byte*)"":NULL;
+    fil->sha1=DB_SHA1&fil->attr?(byte*)"":NULL;
+    fil->tiger=DB_TIGER&fil->attr?(byte*)"":NULL;
+    fil->rmd160=DB_RMD160&fil->attr?(byte*)"":NULL;
 #ifdef WITH_MHASH
-    fil->crc32=DB_CRC32&attr?(byte*)"":NULL;
-    fil->crc32b=DB_CRC32B&attr?(byte*)"":NULL;
-    fil->gost=DB_GOST&attr?(byte*)"":NULL;
-    fil->haval=DB_HAVAL&attr?(byte*)"":NULL;
+    fil->crc32=DB_CRC32&fil->attr?(byte*)"":NULL;
+    fil->crc32b=DB_CRC32B&fil->attr?(byte*)"":NULL;
+    fil->gost=DB_GOST&fil->attr?(byte*)"":NULL;
+    fil->haval=DB_HAVAL&fil->attr?(byte*)"":NULL;
 #endif
   }
   listp=list_append(listp,(void*)fil);
@@ -425,6 +525,7 @@ int check_list_for_match(list* rxrlist,char* text,int* attr)
   for(r=rxrlist;r;r=r->next){
     if((retval=regexec((regex_t*)((rx_rule*)r->data)->crx,text,0,0,0))==0){
       *attr=((rx_rule*)r->data)->attr;
+        error(231,"Matches rule from line #%ld: %s\n",((rx_rule*)r->data)->conf_lineno,((rx_rule*)r->data)->rx);
       break;
     }
   }
@@ -500,42 +601,51 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
   list* r=NULL;
   seltree* a=NULL;
   DIR* dirh=NULL;
-  struct dirent* entp=NULL;
-  struct dirent** resp=NULL;
+  struct AIDE_DIRENT_TYPE* entp=NULL;
+  struct AIDE_DIRENT_TYPE** resp=NULL;
   int rdres=0;
   int addfile=0;
   char* fullname=NULL;
   int e=0;
   int matchattr=attr;
-  #ifndef HAVE_READDIR_R
+#  ifndef HAVE_READDIR_R
   long td=-1;
-  #endif
+#  endif
   int addok=RETOK;
 
+  /* Root is special and it must be checked on it's own. */
+  if( tree->path[0]=='/' && tree->path[1]=='\0' ){
+    addfile=check_node_for_match(tree,"/",0,&matchattr);
+    if(addfile){
+      error(240,"%s match=%d\n",fullname, addfile);
+      file_lst=add_file_to_list(file_lst,"/",matchattr,&addok);
+    }
+  }
+
   if(!(dirh=opendir(tree->path))){
     error(5,"traverse_tree():%s: %s\n", strerror(errno),tree->path);
     return file_lst;
   }
   
-  #ifdef HAVE_READDIR_R
-  resp=(struct dirent**)
-    malloc(sizeof(struct dirent)+_POSIX_PATH_MAX);
-  entp=(struct dirent*)
-    malloc(sizeof(struct dirent)+_POSIX_PATH_MAX);
+#  ifdef HAVE_READDIR_R
+  resp=(struct AIDE_DIRENT_TYPE**)
+    malloc(sizeof(struct AIDE_DIRENT_TYPE)+_POSIX_PATH_MAX);
+  entp=(struct AIDE_DIRENT_TYPE*)
+    malloc(sizeof(struct AIDE_DIRENT_TYPE)+_POSIX_PATH_MAX);
   
-  for(rdres=readdir_r(dirh,entp,resp);
+  for(rdres=AIDE_READDIR_R_FUNC(dirh,entp,resp);
       (rdres==0&&(*resp)!=NULL);
-      rdres=readdir_r(dirh,entp,resp)){
-  #else
-  # ifdef HAVE_READDIR
-  for(entp=readdir(dirh);
+      rdres=AIDE_READDIR_R_FUNC(dirh,entp,resp)){
+#  else
+#   ifdef HAVE_READDIR
+  for(entp=AIDE_READDIR_FUNC(dirh);
       (entp!=NULL&&td!=telldir(dirh));
-      entp=readdir(dirh)){
+      entp=AIDE_READDIR_FUNC(dirh)){
     td=telldir(dirh);
-  # else
-  #  error AIDE needs readdir or readdir_r
-  # endif
-  #endif
+#   else
+#    error AIDE needs readdir or readdir_r
+#   endif
+#  endif
 
     if(strncmp(entp->d_name,".",1)==0){
       if(strncmp(entp->d_name,".",strlen(entp->d_name))==0)
@@ -555,15 +665,17 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
     }
     strncpy(fullname+strlen(tree->path)+e,entp->d_name,strlen(entp->d_name));
     fullname[(strlen(tree->path)+e+strlen(entp->d_name))]='\0';
-    error(230,"Checking %s for match\n",fullname);
+    error(230,_("Checking %s for match\n"),fullname);
     if(attr){ /* This dir and all its subs are added by default */
       addfile=1;
       
       addfile=check_node_for_match(tree,fullname,addfile,&matchattr);
       
       if(addfile){
+        error(240,"%s match=%d\n",fullname, addfile); 
+
 	file_lst=add_file_to_list(file_lst,fullname,matchattr,&addok);
-	if(addfile!=2 && addok!=RETFAIL){
+	if( !(addfile&2) && addok!=RETFAIL){
 	  if(S_ISDIR(((db_line*)file_lst->header->tail->data)->perm_o)){
 	    a=get_seltree_node(tree,
 			       ((db_line*)file_lst->header->tail->data)
@@ -571,14 +683,13 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
 	    if(a==NULL){
 	      a=new_seltree_node(tree,
 				 ((db_line*)file_lst->header->tail->data)
-				 ->filename,0);
+				 ->filename,0,NULL);
 	    }
 	    file_lst=traverse_tree(a,file_lst,attr);
 	  }
 	}
       } else {
-	error(230,"File %s does not match\n",fullname);
-	free(fullname);
+	error(230,_("File %s does not match\n"),fullname);
       }
     } else{ /* This dir is not added by default */
       addfile=0;
@@ -586,6 +697,7 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
       addfile=check_node_for_match(tree,fullname,addfile,&matchattr);
       
       if(addfile){
+        error(240,"%s match=%d\n",fullname, addfile); 
 	file_lst=add_file_to_list(file_lst,fullname,matchattr,&addok);
 	if(addfile!=2 && addok!=RETFAIL){
 	  if(S_ISDIR(((db_line*)file_lst->header->tail->data)->perm_o)){
@@ -595,28 +707,27 @@ list* traverse_tree(seltree* tree,list* file_lst,int attr)
 	    if(a==NULL){
 	      a=new_seltree_node(tree,
 				 ((db_line*)file_lst->header->tail->data)
-				 ->filename,0);
+				 ->filename,0,NULL);
 	    }
 	    file_lst=traverse_tree(a,file_lst,matchattr); 
 	  }
 	}
       } else {
-	error(230,"File %s does not match\n",fullname);
-	free(fullname);
+	error(230,_("File %s does not match\n"),fullname);
       }
       
     }
-    
+    free(fullname);
   }
 
   if(closedir(dirh)==-1){
     error(0,"Closedir() failed for %s\n",tree->path);
   };
   
-  #ifdef HAVE_READDIR_R
+#  ifdef HAVE_READDIR_R
   free(resp);
   free(entp);
-  #endif
+#  endif
   
   /* All childs are not necessarily checked yet */
   if(tree->childs!=NULL){
@@ -636,7 +747,7 @@ list* gen_list(list* prxlist,list* nrxlist,list* erxlist)
   list* r=NULL;
   seltree* tree=NULL;
 
-  tree=new_seltree_node(NULL,"/",0);
+  tree=new_seltree_node(NULL,"/",0,NULL);
 
   gen_seltree(prxlist,tree,'s');
   gen_seltree(nrxlist,tree,'n');
