@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000 Rami Lehti,Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/list.c,v 1.13 2000/04/20 12:44:59 rammer Exp $
+ * $Header: /cvs-root-aide/aide/src/list.c,v 1.14 2000/05/29 08:38:59 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +21,9 @@
 #include <stdlib.h>
 #include "list.h"
 #include "report.h"
+/*for locale support*/
+#include "locale-aide.h"
+/*for locale support*/
 
 /* list
  * limitations:
