@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000 Rami Lehti,Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/symboltable.c,v 1.4 2000/04/20 12:44:59 rammer Exp $
+ * $Header: /cvs-root-aide/aide/src/symboltable.c,v 1.5 2000/05/29 08:38:59 rammer Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,6 +20,9 @@
 
 #include <stdio.h>
 #include "symboltable.h"
+/*for locale support*/
+#include "locale-aide.h"
+/*for locale support*/
 
 list* list_find(char* s,list* item){
 
