@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
  * Copyright (C) 1999,2000 Rami Lehti, Pablo Virolainen
- * $Header: /cvs-root-aide/aide/src/util.c,v 1.17 2000/04/20 12:44:59 rammer Exp $
+ * $Header: /cvs-root-aide/aide/src/util.c,v 1.24 2002/02/10 16:46:01 pablo Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,6 +24,9 @@
 #include <sys/stat.h>
 #include <signal.h>
 #include <ctype.h>
+/*for locale support*/
+#include "locale-aide.h"
+/*for locale support*/
 
 
 #ifndef MAXHOSTNAMELEN
@@ -35,16 +38,16 @@
 #include "util.h"
 #include "aide.h"
 
-#define URL_UNSAFE " <>\"#%{}|\\^~[]`@:\033"
+#define URL_UNSAFE " <>\"#%{}|\\^~[]`@:\033'"
 #define ISPRINT(c) (isascii(c) && isprint(c))
 
 static const char* url_name[] = { 
-  "file", "stdin", "stdout", "stderr", "fd", "sql" };
+  "file", "stdin", "stdout", "stderr", "fd", "sql", "syslog" };
 
 static const int url_value[] = {
-  url_file, url_stdin, url_stdout,url_stderr,url_fd, url_sql };
+  url_file, url_stdin, url_stdout,url_stderr,url_fd, url_sql, url_syslog };
 
-const int url_ntypes=sizeof(url_value)/sizeof(int);
+const int url_ntypes=sizeof(url_value)/sizeof(URL_TYPE);
 
 int cmpurl(url_t* u1,url_t* u2)
 {
@@ -281,6 +284,7 @@ char* perm_to_char(mode_t perm)
       pc[6]='l';
     }
   }
+#if defined (S_ISVTX) && defined (S_IXOTH)
   if((S_ISVTX&perm)==S_ISVTX){
     if((S_IXOTH&perm)==S_IXOTH){
       pc[9]='t';
@@ -288,6 +292,7 @@ char* perm_to_char(mode_t perm)
       pc[9]='T';
     }
   }
+#endif
 
   error(230,"perm_to_char(): %i -> %s\n",perm,pc);
 
@@ -316,6 +321,7 @@ void sig_handler(int signal)
       conf->catch_mmap=0;
     } else {
       error(0,"Caught SIGBUS/SEGV. Exiting\n");
+      abort();
       exit(1);
     }
     break;
@@ -344,3 +350,26 @@ void sig_handler(int signal)
 
   return;
 }
+
+/* We need these dummy stubs to fool the linker into believing that
+   we do not need them at link time */
+
+void* dlopen(char*filename,int flag)
+{
+  return NULL;
+}
+
+void* dlsym(void*handle,char*symbol)
+{
+  return NULL;
+}
+
+void* dlclose(void*handle)
+{
+  return NULL;
+}
+
+const char* dlerror(void)
+{
+  return NULL;
+}
